<?php

//  Copyright (C) 2002-2006 Balint Kis (balint@k-i-s.net)
//  Copyright (C) 2005-2007 Solomon Peachy (pizza@shaftnet.org)

//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.

//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

include_once "calendar.php";

function xml_to_html($xml_code) {
  $xml_code = ereg_replace("<", "&lt;", $xml_code);
  $xml_code = ereg_replace(">", "&gt;", $xml_code);
  $xml_code = ereg_replace('\"', "&quot;", $xml_code);
  $xml_code = ereg_replace("\n", "<br/>\n", $xml_code);
  return $xml_code;
}

function emit_a($ref, $value, $attrs = "") {
  return "<a href=\"$ref\" $attrs >$value</a>\n";
}

function emit_td($value, $attrs = "") {
  return "<td $attrs >$value</td>\n";
}

function emit_th($value, $attrs = "") {
  return "<th $attrs >$value</th>\n";
}

function emit_option($value, $text, $selected, $disabled = FALSE) {
  $sel = "";

  if ($selected === TRUE) 
    $sel = ' selected="selected" ';
  
  if ($disabled === TRUE)
    $sel .= ' disabled="disabled" ';

  return "<option value=\"$value\" $sel>$text</option>\n";
}

function emit_float_combo($identifier, $current_value, 
			  $min_value, $max_value, $step) {
  global $strings;

  $rval = "<select name=\"$identifier\">";
  for ($i = $min_value; $i <= $max_value; $i += $step) {
    $v = number_format($i, 2, $strings['formats']['decimal'], $strings['formats']['thousands']);
    if (!bccomp($i, $current_value, 2)) {
      $rval .= emit_option($i, $v, TRUE);
    } else {
      $rval .= emit_option($i, $v, FALSE);
    }

  }
  $rval .= "</select>";
  return $rval;
}

function get_path_to_folder($database, $folder_id) {
  global $strings;

  $stack_size = 0;

  $folder_data = pg_fetch_assoc(pg_query($database, "
      select parent_folder, caption
        from folder
       where identifier = '$folder_id'"));
  $folder_id = $folder_data['parent_folder'];

  if ($folder_id == "")
    return;

  do {
    $folder_data = pg_fetch_assoc(pg_query($database, "
      select parent_folder, caption
        from folder
       where identifier = '$folder_id'"));
    $folder_data['caption'] = strip_tags($folder_data['caption']);
    $folder_data['caption'] = makeZanza($folder_data['caption'], 
$strings['photo']['no_title']);
    $stack[$stack_size++] = emit_a("folder.php?folder=$folder_id", $folder_data['caption']);
    $folder_id = $folder_data['parent_folder'];
  } while ($folder_id);

  $path = "";
  for ($i = $stack_size - 1; $i >= 0; $i--) {
    $path = $path . " : $stack[$i]";
  }

  return $path;
}

function get_path_to_album($database, $album_id) {
  global $strings;

  $stack_size = 0;

  $album_data = pg_fetch_assoc(pg_query($database, "
      select parent_album, caption
        from album
       where identifier = '$album_id'"));
  $album_id = $album_data['parent_album'];

  if ($album_id == "")
    return;

  do {
    $album_data = pg_fetch_assoc(pg_query($database, "
      select parent_album, caption
        from album
       where identifier = '$album_id'"));
    $album_data['caption'] = strip_tags($album_data['caption']);
    $album_data['caption'] = makeZanza($album_data['caption'], $strings['photo']['no_title']);
    $stack[$stack_size++] = "<a href=\"album.php?album=$album_id\">$album_data[caption]</a>";
    $album_id = $album_data['parent_album'];
  } while ($album_id);

  $path = "";
  for ($i = $stack_size - 1; $i >= 0; $i--) {
    $path = $path . " : $stack[$i]";
  }

  return $path;
}

function get_photo_title($caption, $title) {
  global $strings;

  $new_title = $title ? $title : $caption;
  if (!$new_title)
    $new_title = $strings['photo']['no_title'];

  return $new_title;
}

function display_storage_location_combo($identifier, $current_value) {
  global $strings;

  print "<select name=\"$identifier\">";
  print emit_option("TRAY", $strings['generic']['tray'], $current_value == "tray");
  print emit_option("ALBUM", $strings['generic']['album'], $current_value == "album");
  print emit_option("CDR", $strings['generic']['cdr'], $current_value == "cdr");
  print emit_option("null", $strings['generic']['none'], ($current_value != "tray" && $current_value != "album" && $current_value != "cdr"));
  print "</select>";
}

function display_photos_per_page_combo($identifier, $current_value, $wrapped=TRUE) {
  global $thm_elem;
  global $strings;

  $max_row = 200;
  if ($wrapped) {
    print "<table>\n<tr><td>";
    print $thm_elem['button.per.page'];
    print "</td>\n<td>\n";
  }
  print "<select name=\"$identifier\">";
  for ($i=5; $i < ($max_row + 1); $i+=5) {
    print emit_option($i, $i, $i == $current_value);
  }
  print emit_option("3000", $strings['generic']['display_all'], $current_value == 3000);
  print "</select>";
  if ($wrapped) {
    print "</td></tr></table>";
  }
}

function display_print_format_combo($identifier, $current_value) {
  print "<select name=\"$identifier\">";
  print emit_option("ps", "PS - Postscript", (strncmp($current_value, "ps", 2) == 0));
  print emit_option("pdf", "PDF - Portable Document Format", (strncmp($current_value, "pdf", 3) == 0));
  print "</select>";
}

function display_popup_delay_combo($identifier, $popup_delay, $popup_enabled) {
  global $strings;

  print "<select name=\"$identifier\">";
  print emit_option('f', $strings['generic']['disabled'], $popup_enabled == 'f');

  for ($i=100; $i <= 2000; $i+=100) {
    print emit_option($i, "$i ".$strings['generic']['msec'], $i == $popup_delay);
  }
  print "</select>";
}

function display_first_day_combo($identifier, $current_value) {
  global $dayNames;
  
  print "<select name=\"$identifier\">";
  for ($i = 0; $i < 7 ; $i++) {
    print emit_option($i, $dayNames[$i], $current_value == $i);
  }
  print "</select>";
}

function display_search_engine_type_combo($identifier, $current_value) {
  global $strings;
  global $cbir_host;
  global $cbir_port;

  print "<select name=\"$identifier\">";
  print emit_option("1", $strings['search']['gen_text'], $current_value == 1);
  print emit_option("2", $strings['search']['adv_text'], $current_value == 2);
  if ($cbir_host != "" && $cbir_port != "") {
    print emit_option("3", $strings['search']['img_layout'], $current_value == 3);
    print emit_option("4", $strings['search']['img_color'], $current_value == 4);
  }
  print "</select>";
}

function display_colorspace_combo($identifier, $current_value) {
  global $strings;

  print "<select name=\"$identifier\">";
  if ($current_value == -1) {
    print emit_option("", "", TRUE);
  }
  print emit_option("1", "sRGB", $current_value == 1);
  print emit_option("2", "AdobeRGB1998", $current_value == 2);
  print emit_option("99", $strings['generic']['embedded'], $current_value == 99);
  print "</select>";
}

function display_photo_index_status($offset, $limit, $num_of_items) {
  $first_index = $offset + 1;
  $last_index = $offset + $limit;

  if ($last_index > $num_of_items)
    $last_index = $num_of_items;

  if (!$limit ||($limit == $num_of_items)) {
    if ($first_index < $last_index)
      return "<b>$first_index</b>-<b>$last_index</b>";
    else
      return "<b>$num_of_items</b>";
  }
  else
    return "<b>$first_index</b>-<b>$last_index</b> of <b>$num_of_items</b>";
}

function emit_yes_no_text($value) {
  global $strings;

  if ($value == 't')
    return $strings['generic']['yes'];
  else
    return $strings['generic']['no'];
}

function display_yes_no_combo($item_name, $value) {
  print emit_yes_no_combo($item_name, $value);
}

function emit_yes_no_combo($item_name, $value) {
  global $strings;
  $val = "";

  $val .= "<select name=\"$item_name\">\n";
  $val .= emit_option('t', $strings['generic']['yes'], $value == 't');
  $val .= emit_option('f', $strings['generic']['no'], $value == 'f');
  $val .= "</select>\n";
  return $val;
}

function emit_filetype_combo($item_name, $value) {
  global $strings;
  $val = "";

  $val .= "<select name=\"$item_name\">\n";
  $val .= emit_option('jpeg', "JPEG", $value != 'png');
  $val .= emit_option('png', "PNG", $value == 'png');
  $val .= "</select>\n";
  return $val;
}

function emit_enable_disable_combo($item_name, $value) {
  global $strings;

  $val = "<select name=\"$item_name\">\n";
  
  $val .= emit_option('t', $strings['generic']['enabled'], $value == 't');
  $val .= emit_option('f', $strings['generic']['disabled'], $value == 'f');
  $val .= "</select>\n";
  return $val;
}

function orientation_to_name($database, $orientation) {
  global $strings;
  
  $orients = pg_query($database, "select name from orientations where identifier = $orientation");
  if (pg_num_rows($orients)) {
    $orients = pg_fetch_assoc($orients);
    return $orients['name'];
  }
  return $strings['generic']['unknown']." ($orientation)";
}

function orientation_to_xform($database, $orientation) {
  global $strings;

  $orients = pg_query($database, "select im_transform from orientations where identifier = $orientation");
  if (pg_num_rows($orients)) {
    $orients = pg_fetch_assoc($orients);
    return $orients['im_transform'];
  }
  return "";
}

function orientation_id_from_string($database, $name) {
    $orients = pg_query($database, "select identifier from orientations where name = '$name'");
	if (pg_num_rows($orients)) {
		$orients = pg_fetch_assoc($orients);
		return $orients['identifier'];
	}
	return "1";
}

function display_access_combo($item_name, $value, $kind) {
  global $strings;

  print "<select name=\"$item_name\">\n";
  if ($kind == "null") {
    print emit_option("null", $strings['generic']['none'], (!$value));
  }
  print emit_option("1", $strings['generic']['public'], $value == "1");
  print emit_option("2", $strings['generic']['protected'], $value == "2");
  print emit_option("3", $strings['generic']['private'], $value == "3");
  print "</select>\n";
}

function display_hide_original_combo($item_name, $value) {
  global $strings;

  print "<select name=\"$item_name\">\n";
  print emit_option("f", $strings['generic']['access_auth_users'], $value == 'f');
  print emit_option("t", $strings['generic']['access_owner_only'], $value == 't');
  print "</select>\n";
}

function display_download_counter_combo($item_name, $value) {
  print "<select name=\"$item_name\">\n";
  for ($pcs = 0; $pcs < 101; $pcs++) {
    print emit_option("$pcs", "$pcs", $pcs == $value);
  }
  print "</select>\n";
}

function display_download_expiry_date_combo($item_name, $value) {
  global $strings;

  print "<select name=\"$item_name\">\n";
  print emit_option("1", "1 ".$strings['generic']['day'], $value == 1);
  print emit_option("2", "2 ".$strings['generic']['days'], $value == 2);
  print emit_option("3", "3 ".$strings['generic']['days'], $value == 3);
  print emit_option("4", "4 ".$strings['generic']['days'], $value == 4);
  print emit_option("5", "5 ".$strings['generic']['days'], $value == 5);
  print emit_option("6", "6 ".$strings['generic']['days'], $value == 6);
  print emit_option("7", "1 ".$strings['generic']['week'], $value == 7);
  print emit_option("14", "2 ".$strings['generic']['week'], $value == 14);
  print emit_option("21", "3 ".$strings['generic']['weeks'], $value == 21);
  print emit_option("30", "1 ".$strings['generic']['month'], $value == 30);
  print emit_option("61", "2 ".$strings['generic']['months'], $value == 61);
  print emit_option("91", "3 ".$strings['generic']['months'], $value == 91);
  print emit_option("182", "6 ".$strings['generic']['months'], $value == 182);
  print emit_option("365", "1 ".$strings['generic']['year'], $value == 365);
  print emit_option("730", "2 ".$strings['generic']['years'], $value == 730);
  print emit_option("1095", "3 ".$strings['generic']['years'], $value == 1095);
  print "</select>\n";
}

function display_photo_navigator($hyperlink, $offset, $limit, $num_of_items) {
  global $thm_elem;
  global $strings;
  global $po_options;

  if ($limit <= 0)  
    $limit = 1;

  if (($offset > 0) || 
      ($limit < $num_of_items)) {
    $last = $limit * floor($num_of_items/$limit);

    $previous = $offset - $limit;
    $next = $offset + $limit;

    if ($previous < 0)
      $previous = 0;

    if ($next > $last)
      $next = $last;

    print "<div class=\"photo_nav\">";
    print sprintf($thm_elem['button.first'], "$hyperlink&amp;offset=0", $strings['generic']['page']);
    print sprintf($thm_elem['button.previous'], "$hyperlink&amp;offset=$previous", $strings['generic']['page'], "");

    for ($i=0; $i < ceil($num_of_items/$limit); $i++) {
      $aux=$i*$limit;
      $aux1 = $aux + 1;
      $aux2=($i+1)*$limit;
      if ($aux2 > $num_of_items) $aux2 = $num_of_items;

      if ($aux == $offset)	
	print sprintf($thm_elem['button.option.a'], "$hyperlink&amp;offset=$aux", "$aux1-$aux2");

      else
	print sprintf($thm_elem['button.option'], "$hyperlink&amp;offset=$aux", "$aux1-$aux2");
    }
    print sprintf($thm_elem['button.next'], "$hyperlink&amp;offset=$next", $strings['generic']['page'], "");
    print sprintf($thm_elem['button.last'], "$hyperlink&amp;offset=$last", $strings['generic']['page']);
    print "</div>";
  } 
}

function display_moveto_folder_entry($folder_data, $folder_id, $shift, $disabled) {

  $zanza = substr($folder_data[1], 0, 25);
  $zanza = htmlspecialchars($zanza);

  for ($i = 0; $i < $shift; $i++) {
    $zanza = "&nbsp;&nbsp;" . $zanza;
  }

  print emit_option($folder_data[0], $zanza, $folder_data[0] == $folder_id, $disabled);
}

function display_moveto_recursive($folder_tmp, $folder_data, $folder_id, $shift, $disabled) {
  foreach($folder_tmp as $folder_sub_data) {
    if ($folder_sub_data[2] == $folder_data[0]) {
      display_moveto_folder_entry($folder_sub_data, $folder_id, $shift, $disabled);
      display_moveto_recursive($folder_tmp, $folder_sub_data, $folder_id, $shift + 1, $disabled);
    }
  }
}

function display_folder_actions($folder_all, $folder_id) {
  global $thm_elem;
  global $po_user;

  print "<br/>";
  print "<table>\n";
  print "<tr><td>\n";
  print "<input type=\"hidden\" name=\"source_folder\" value=\"$folder_id\"/>\n";
  
  print $thm_elem['button.move.to.f'];
  print $thm_elem['button.go.to.f'];

  print "</td><td>\n";

  print "<select name=\"destination_folder\">\n";
  print emit_option("", "&nbsp;", TRUE);
  if ($folder_all != "") {
    $folder_tmp = array();
    for ($i=0; $i < pg_num_rows($folder_all); $i++) {
      $folder_tmp[$i] = pg_fetch_array($folder_all, $i);
    }
    foreach($folder_tmp as $folder_data) {
      if ($folder_data[0] == $po_user['trash_folder']) continue;

      if ($folder_data[2] == null) {
        display_moveto_folder_entry($folder_data, $folder_id, 0, FALSE);
        display_moveto_recursive($folder_tmp, $folder_data, $folder_id, 1, FALSE);
      }
    }
  }
  print "</select>\n";
  print "</td>\n</tr>\n</table>\n";
}

function display_album_actions($album_all, $album_id) {
  global $strings;
  global $thm_elem;

  print "<br/>";
  print "<table>\n";
  print "<tr>\n<td>\n";
  if (isset($album_id) && $album_id != '') {
    print $thm_elem['button.move.to.a'];
  }
  print $thm_elem['button.link.to.a'];
  print $thm_elem['button.go.to.a'];

  print "<input type=\"hidden\" name=\"move_source_album\" value=\"$album_id\"/>";
  print "</td><td>\n";
  print "<select name=\"destination_album\">\n";
  print emit_option("", "&nbsp;", TRUE);
  if ($album_all != "") {
    $album_tmp = array();
    for ($i=0; $i < pg_num_rows($album_all); $i++) {
      $album_tmp[$i] = pg_fetch_array($album_all, $i);
    }

    foreach($album_tmp as $album_data) {
      if ($album_data[2] == null) {
        display_moveto_folder_entry($album_data, $album_id, 0, FALSE);
        display_moveto_recursive($album_tmp, $album_data, $album_id, 1, FALSE);
      }
    }
  }

  print "</select>\n";
  print "</td>\n</tr>\n</table>\n";
}

function display_list_slide_option($type, $url) {
  global $thm_elem;

  if ($type == 'slide') {
    print sprintf($thm_elem['button.list.view'],"$url&amp;photo_view=list");
  } else {
    print sprintf($thm_elem['button.slide.view'],"$url&amp;photo_view=slide");
  }
}

function photos_in_spool($database) {
  global $po_user;

  $photos_in_spooler = pg_fetch_row(pg_query($database, "select count(distinct(version)) from album_content where album = $po_user[spool_album]"));
  $num_photos = $photos_in_spooler[0];

  return $num_photos;
}

function display_spool_actions($database, $userid) {
  global $po_user;
  global $strings;

  $num_photos = photos_in_spool($database);

  if ($num_photos <= 0)  return;

  print "<br/>";
  print "<table><tr><td><b>".$strings['spool']['photos_in'].": $num_photos&nbsp;</b></td><td><a href=\"album.php?album=$po_user[spool_album]\">".$strings['spool']['view']."</a> | <a href=\"spool.empty.php\">".$strings['spool']['empty']."</a> | <a href=\"my.tools.php?selector=1&amp;type=spool\">".$strings['generic']['bulk_update']."</a></td>";
  print "<td>";
  display_printto();
  print "</td><td>";
  display_export();
  print "</td></tr></table>";
}

function display_printto() {
  global $strings;
  global $thm_elem;

  print "<table border=\"0\" cellspacing=\"0\" cellpadding=\"0\">\n";
  print "<tr>\n<td align=\"left\" valign=\"middle\">\n";
  print $thm_elem['button.print'];
  print "</td>\n<td align=\"left\" valign=\"middle\" >\n";
  print "<select name=\"print_type\">\n";
  print emit_option("", '&nbsp;', TRUE);
  print emit_option("label", $strings['generic']['label'], FALSE);
  print emit_option("brochure", $strings['generic']['brochure'], FALSE);
  print "</select>\n";
  print "</td>\n</tr>\n</table>\n";
}

function display_export() {
  global $thm_elem;
  global $strings;

  print "<table border=\"0\" cellspacing=\"0\" cellpadding=\"0\">\n";
  print "<tr>\n<td align=\"left\" valign=\"middle\">\n";
  print $thm_elem['button.export'];
  print "</td>\n<td align=\"left\" valign=\"middle\" >\n";
  print "<select name=\"export_type\">\n";
  print emit_option("", '&nbsp;', TRUE);
  print emit_option("photo", $strings['generic']['photos_only'], FALSE);
  print emit_option("photo_xml", $strings['generic']['photos_xml'], FALSE);
  
  // print emit_option("photo_iptcl", $strings['generic']['photos_iptc'], FALSE);
  // print emit_option("photo_iptc_xml", $strings['generic']['photos_iptc_xml'], FALSE);

  print "</select>\n";
  print "</td>\n</tr>\n</table>\n";
}

function display_transform($photo_id) {
  global $strings;
  global $thm_elem;

  print "<table border=\"0\" cellspacing=\"0\" cellpadding=\"0\">\n";
  print "<tr>\n<td align=\"left\" valign=\"middle\">\n";
  print $thm_elem['button.rotate'];
  print "</td>\n<td align=\"left\" valign=\"middle\" >\n";
  print "<select name=\"transform\">\n";
  print emit_option("", '&nbsp;', TRUE);
  print emit_option("r90", $strings['generic']['rot_90'], FALSE);
  print emit_option("r180", $strings['generic']['rot_180'], FALSE);
  print emit_option("r270", $strings['generic']['rot_270'], FALSE);
  print emit_option("flip", $strings['generic']['flip'], FALSE);
  print emit_option("flop", $strings['generic']['flop'], FALSE);
  print "</select>\n";
  print "</td>\n</tr>\n</table>\n";
}

/* returns one row of the tooltip table */
function tooltip_row($name, $value) {
  if ($value == "") {
	return "";
  }
  $name = htmlentities($name, ENT_COMPAT, "UTF-8");
  $value = htmlentities($value, ENT_COMPAT, "UTF-8");
  return "<tr><td>$name :</td><td>$value</td></tr>";
}

function display_photo_slides($display_type, $container_id, $photo_data, $offset, $num_of_items, $framed, $checks = FALSE) {
  global $po_options;
  global $po_user;
  global $strings;
  global $access;
  global $tooltip_width;

  $my_user_id = $po_user['id'];
  $my_user_type = $po_user['type'];

  if ($my_user_type < PO_USER_TYPE_CLIENT) 
    $checks = FALSE;

  if ($display_type == "album") {
    $option = "&amp;album=$container_id";
  } else {
    $option = "";
  }

  print "<div class=\"photo_slides\">\n";

  $checkboxes = 0;

  for ($cell_counter = 0; $cell_counter < $num_of_items; $cell_counter++) {

    $photo = pg_fetch_assoc($photo_data, $offset + $cell_counter);
    if ($photo == FALSE) break;

    $photo['date_of_exposure'] = substr($photo['date_of_exposure'], 0, 10);
    $photo['date_added'] = substr($photo['date_added'], 0, 10);
    if ($photo['place'] != "" && ($photo['city'] != "" || $photo['state'] != "" || $photo['country'] != "")) { $photo['place'] .= ","; }
    if ($photo['city'] != "" && ($photo['state'] != "" || $photo['country'] != "")) { $photo['city'] .= ","; }
    if ($photo['state'] != "" && ($photo['country'] != "")) { $photo['state'] .= ","; }

    if ($photo['place'] == 'Unknown') {
      $photo['location'] = "";
    } else {
      $photo['location'] = "$photo[place] $photo[city] $photo[state] $photo[country]";
    }

    if ($framed) {
      display_slide_top($cell_counter, $checks);
    }
    
    if ($po_options['popup_enable'] == 't') {
      $tt_id = "tt_img_$photo[identifier]_$photo[version]";
      print "<div class=\"hide_tooltip\" id=\"$tt_id\"><table class=\"tooltip\">";
      print tooltip_row($strings['generic']['title'], $photo['title']);
      print tooltip_row($strings['generic']['author'], $photo['author']);
      if ($photo['caption']) {
	print tooltip_row($strings['generic']['caption'],  $photo['caption']);
      }
      if ($photo['location']) {
	print tooltip_row($strings['generic']['location'], $photo['location']);
      }
      if ($photo['date_of_exposure'] != "")
	print tooltip_row($strings['generic']['exposed'], $photo['date_of_exposure']);
      else
	print tooltip_row($strings['generic']['created'], $photo['date_added']);
      if ($my_user_id != FALSE ) {
	print tooltip_row($strings['generic']['access'], $strings['generic'][$access[$photo['access_rights']]]);
      }
      if ($photo['rating'] > 0)
	print tooltip_row($strings['generic']['rating'], round(11 - $photo['rating'], 3));
      print tooltip_row($strings['generic']['views'], $photo['views']);
      print "</table></div>\n";
    }

    print "<a class=\"nodecor\" href=\"photo.php?photo=$photo[identifier]&amp;ver=$photo[version]$option\" ";
    if ($po_options['popup_enable'] == 't') {
      print "onmouseover=\"TagToTip('$tt_id', DELAY, $po_options[popup_delay], COPYCONTENT, false, BORDERWIDTH, 0, PADDING, 0, WIDTH, $tooltip_width, BGCOLOR,'transparent');\"";
    }
    if ($framed) 
      $class="class=\"slide\"";
    else 
      $class="";

    print "><img $class src=\"image.display.php?image=$photo[identifier]&amp;ver=$photo[version]&amp;size=1\" alt=\"# $photo[identifier]:$photo[version]\"/></a>\n";

    if ($framed) {
      display_frame_bottom($cell_counter, "$photo[identifier]:$photo[version]", $checks);
    }

    if ($checks) {
      $checkboxes++;
    }
  }

  print "</div> <!-- slides end --> \n";

  return $checkboxes;
}

function display_photo_list($display_type, $container_id, $photo_data, $offset, $num_of_items) {
  global $po_options;
  global $po_user;
  global $strings;
  global $access;

  $checkboxes = 0;

  $my_user_id = $po_user['id'];
  $my_user_type = $po_user['type'];

  $option = "";

  if ($display_type == "album") {
    $option = "&amp;album=$container_id";
  } 

  print "<table class=\"folder\">\n";
  print "<tr class=\"folder_top\"><td colspan=\"5\">Photos</td></tr>\n";
  print "<tr>\n";
  if ($my_user_id != FALSE ) {
    print "<th width=\"4%\"></th>\n";
  }
  print "<th width=\"56%\">".$strings['generic']['caption']."</th>\n";
  print "<th width=\"10%\">".$strings['generic']['rating']."</th>\n";
  if ($my_user_id != FALSE ) {
    print "<th width=\"6%\">".$strings['generic']['access']."</th>\n";
  }
  print "<th width=\"20%\">".$strings['generic']['exposed']."</th>\n";
  print "</tr>\n";

  for ($photo_counter = 0; $photo_counter < $num_of_items; $photo_counter++) {
    $photo = pg_fetch_assoc($photo_data, $offset + $photo_counter);
    if ($photo == FALSE) break;
    print "<tr>";
    if ($my_user_id != FALSE ) {
      print "<td>\n";
      print "<input type=\"checkbox\" name=\"selection[]\" id=\"cb$photo_counter\" value=\"$photo[identifier]:$photo[version]\"/>\n";
      print "</td>\n";
      $checkboxes++;
    }
    $photo['caption'] = makeZanza($photo['caption'], $strings['photo']['no_title']);

    print "<td>";

    if ($po_options['popup_enable'] == 't') {
      $tt_id = "tt_img_$photo[identifier]_$photo[version]";
      print "<div class=\"hide_tooltip\" id=\"$tt_id\">";
      print "<img src=\"image.display.php?image=$photo[identifier]&amp;ver=$photo[version]&amp;size=1\" alt=\"# $photo[identifier]:$photo[version]\" />";
      print "</div>\n";
      $onmouseover = "onmouseover=\"TagToTip('$tt_id', DELAY, $po_options[popup_delay], COPYCONTENT, false, BORDERWIDTH, 0, PADDING, 0);\"";
    } else {
      $onmouseover = "";
    }  

    print "<a href=\"photo.php?photo=$photo[identifier]&amp;ver=$photo[version]$option\" $onmouseover >$photo[caption]</a></td>\n";

    $photo['rating'] = round($photo['rating'], 0);
    if ($photo['rating'] > 0)
      $photo['rating'] = 11 - $photo['rating'];
    else
      $photo['rating'] = "";
    print "<td>$photo[rating]</td>\n";
    if ($my_user_id != FALSE ) {
      print emit_td($strings['generic'][$access[$photo['access_rights']]]);
    }
    if ($photo['date_of_exposure'] != "")
      print emit_td(emit_date_html($photo['date_of_exposure'], 'date'));
    else
      print "<td>".emit_date_html($photo['date_added'], 'date')."</td>\n";
    print "</tr>\n";
  }
  print "</table>\n";
  return $checkboxes;
}

function emit_lens_string($min_aperture, $max_aperture, $min_focal, $max_focal) {
  $rval = "";
  if (!$min_aperture)
    return $rval;

  if ($min_aperture == $max_aperture)
    $rval .= "$min_aperture";
  else
    $rval .= "$min_aperture-$max_aperture";

  if ($min_focal == $max_focal)
    $rval .= "/$min_focal";
  else
    $rval .= "/$min_focal-$max_focal";

  return $rval;
}

function emit_manuf_add_needed($type) {
  global $thm_elem;
  global $strings;

  $rval = "<div align=\"center\">";
  $rval .= sprintf($strings['profile']['must_add_manufacturers'], $strings['generic'][$type]);
  $rval .= "<form action=\"manufacturer.add.php\" method=\"post\" accept-charset=\"".$strings['formats']['encoding']."\">";
  $rval .= $thm_elem['button.add.manufacturer'];
  $rval .= "</div>\n";

  return $rval;
}

function emit_profile_manuf_combo($url, $type, $span, $rows, $filter) {
  global $strings;

  print "<tr class=\"folder_top\">";
  print "<td colspan=\"$span\">\n";
  print $strings['generic'][$type]." ".$strings['generic']['types']."</td>\n";
  print "<td>\n";
  print "<select onchange=\"location = '$url&amp;filter=' + this.options[this.selectedIndex].value;\">\n";
  print emit_option('', $strings['generic']['all'], FALSE);
  for ($i=0; $i < pg_num_rows($rows); $i++) {
    $r = pg_fetch_row($rows, $i);
    print emit_option("$r[1]", $r[1], $r[1] == $filter);
  }
  print "</select>";
  print "</td>";
  print "</tr>";
}

function emit_manufacturer_combo($name, $rows, $current_value) {
  $rval = "<select name=\"$name\">";
  $num = pg_num_rows($rows);
  for ($i=0; $i < $num; $i++) {
    $r = pg_fetch_row($rows, $i);
    $rval .= emit_option($r[0], $r[1], $current_value==$r[1]);
  }

  $rval .= "</select>\n";
  
  return $rval;
}

function get_users_folder($database) {
  global $po_user;
  
  return pg_query($database, "select identifier, caption, parent_folder from folder where users=$po_user[id] and identifier != $po_user[trash_folder] order by date_of_creation desc");
}

function get_users_album($database) {
  global $po_user;
  
  return pg_query($database, "select identifier, caption, parent_album from album where users=$po_user[id] and identifier != $po_user[spool_album] order by date_of_creation desc");
}

function display_folder_combo($identifier, $all_values, $current_value, $kind) {
  global $strings;

  $num = pg_num_rows($all_values);
  print "<select name=\"$identifier\">";

  switch ($kind) {
  case "null":
    print emit_option("null", $strings['generic']['none'], FALSE);
    break;
  case "all":
    print emit_option("all", $strings['generic']['all_folders'], FALSE);
    break;
  case "null+all":
  case "all+null":
    print emit_option("null", $strings['generic']['none'], FALSE);
    print emit_option("all", $strings['generic']['all_folders'], FALSE);
    break;
  default:
    break;
  }

  if ($all_values != "") {
    $folder_tmp = array();
    for ($i=0; $i < pg_num_rows($all_values); $i++) {
      $folder_tmp[$i] = pg_fetch_array($all_values, $i);
    }

    foreach($folder_tmp as $folder_data) {
      if ($folder_data[2] == null) {
        display_moveto_folder_entry($folder_data, $current_value, 0, FALSE);
        display_moveto_recursive($folder_tmp, $folder_data, $current_value, 1, FALSE);
      }
    }
  }
  print "</select>";
}

function display_album_combo($identifier, $all_values, $current_value, $kind) {
  global $strings;

  $num = pg_num_rows($all_values);
  print "<select name=\"$identifier\">";
  switch ($kind) {
  case "null":
    print emit_option("null", $strings['generic']['none'], FALSE);
    break;
  case "all":
    print emit_option("all", $strings['generic']['all_albums'], FALSE);
    break;
  case "null+all":
  case "all+null":
    print emit_option("null", $strings['generic']['none'], FALSE);
    print emit_option("all", $strings['generic']['all_albums'], FALSE);
    break;
  default:
    break;
  }

  if ($all_values != "") {
    $folder_tmp = array();
    for ($i=0; $i < pg_num_rows($all_values); $i++) {
      $folder_tmp[$i] = pg_fetch_array($all_values, $i);
    }

    foreach($folder_tmp as $folder_data) {
      if ($folder_data[2] == null) {
        display_moveto_folder_entry($folder_data, $current_value, 0, FALSE);
        display_moveto_recursive($folder_tmp, $folder_data, $current_value, 1, FALSE);
      }
    }
  }
  print "</select>";
}

function display_combo($identifier, $all_values, $current_value, $separator, $no_null) {
  print emit_combo($identifier, $all_values, $current_value, $separator, $no_null);
}

function emit_combo($identifier, $all_values, $current_value, $separator, $no_null) {
  global $strings;

  $rval = "";

  $num = pg_num_rows($all_values);
  $rval .= "<select name=\"$identifier\">";
  if ($no_null == FALSE) {
    $rval .= emit_option("null", $strings['generic']['none'], FALSE);
  }
  for ($i=0; $i < $num; $i++) {
    $value = pg_fetch_row($all_values, $i);

    if ($value == FALSE) continue;
    if (!isset($value[0])) $value[0] = '';
    if (!isset($value[1])) $value[1] = '';
    if (!isset($value[2])) $value[2] = '';
    if (!isset($value[3])) $value[3] = '';
    if (!isset($value[4])) $value[4] = '';

    if ($value[4] == 'Unknown') $value[4] = $strings['generic']['unknown'];

    if ($value[1] != '' && ($value[2] != '' || $value[3] != '' ||  $value[4] != '')) {
      $value[1] = sprintf("%s%s", $value[1], $separator);
    }
    if ($value[2] != '' && ($value[3] != '' ||  $value[4] != '')) {
      $value[2] = sprintf("%s%s", $value[2], $separator);
    }
    if ($value[3] != '' && $value[4] != '') {
      $value[3] = sprintf("%s%s", $value[3], $separator);
    }
    $rval .= emit_option($value[0], "$value[1] $value[2] $value[3] $value[4]", $value[0] == $current_value);
  }
  $rval .= "</select>";
  return $rval;
}

function display_icc_profiles_combo($item_name, $value) {
  global $icc_profiles;
  global $strings;

  $num_of_icc_profiles = count($icc_profiles);

  print "<select name=\"$item_name\">\n";
  print emit_option("null", $strings['generic']['none'], FALSE);

  for ($i = 1; $i < $num_of_icc_profiles; $i++) {
    print emit_option($i, $icc_profiles[$i]['name'], FALSE);
  }
  print "</select>\n";
}

function display_lens_combo($identifier, $all_values, $current_value, $with_comas, $no_null) {
  global $strings;

  $num = pg_num_rows($all_values);
  print "<select name=\"$identifier\">";
  if ($no_null == FALSE) {
    print emit_option("null", $strings['generic']['none'], FALSE);
  }
  for ($i=0; $i < $num; $i++) {
    $value = pg_fetch_row($all_values, $i);
    if ($with_comas == 1) {
      if ($value[1] != '' && ($value[2] != '' ||  $value[3] != '')) {
        $value[1] = sprintf("%s,", $value[1]);
      }
      if ($value[2] != '' && $value[3] != '') {
        $value[2] = sprintf("%s,", $value[2]);
      }
    }
    print emit_option("$value[0]", "$value[1] $value[2] $value[3] ". emit_lens_string($value[4], $value[5], $value[6], $value[7]), $value[0] == $current_value);
  }
  print "</select>";
}

function display_currency_combo($identifier, $all_values, $current_value, $no_null) {
  global $strings;

  $num = pg_num_rows($all_values);
  print "<select name=\"$identifier\">";
  if ($no_null == FALSE) {
    print emit_option("null", $strings['generic']['none'], FALSE);
  }
  for ($i=0; $i < $num; $i++) {
    $value = pg_fetch_assoc($all_values, $i);
    print emit_option($value['identifier'], "$value[alpha_code] $value[name]", $value['identifier'] == $current_value);
  }
  print "</select>";
}

function display_users($database, $user_list, $viewer_user_id) {
  global $access;
  global $strings;

  $viewer_user_data = FALSE;

  if ($viewer_user_id)
    $viewer_user_data = pg_fetch_assoc(pg_query($database, "
    select identifier, last_name, first_name, member_since, type 
      from users 
     where users.type>".PO_USER_TYPE_CLIENT." 
       and identifier=$viewer_user_id"));
  
  print "<div class=\"folder\">";
  print "<table class=\"folder\">\n";
  print "<tr>\n";
  print "<th width=\"40%\">".$strings['generic']['name']."</th>\n";
  print "<th align=\"center\" width=\"10%\">".$strings['generic']['photos']."</th>\n";
  print "<th align=\"center\" width=\"10%\">".$strings['generic']['folders']."</th>\n";
  print "<th align=\"center\" width=\"10%\">".$strings['generic']['albums']."</th>\n";
  print "<th align=\"center\" width=\"10%\">".$strings['generic']['equipment']."</th>\n";
  print "<th width=\"20%\">".$strings['generic']['member_since']."</th>\n";
  print "</tr>\n";

  if ($viewer_user_data != FALSE) {
    $user_stats = pg_fetch_assoc(pg_query($database, "
   select count_photos_by_user($viewer_user_data[identifier]) as photos,
          count_folders_by_user($viewer_user_data[identifier]) -1 as folders,
          count_albums_by_user($viewer_user_data[identifier]) -1 as albums,
          count_equipment_by_user($viewer_user_data[identifier]) as equipment"));

    print "<tr>";
    print "<td><a href=\"user.php?user=$viewer_user_data[identifier]\">$viewer_user_data[last_name], $viewer_user_data[first_name]</a></td>";
    print "<td align=\"center\">$user_stats[photos]</td>";
    print "<td align=\"center\">$user_stats[folders]</td>";
    print "<td align=\"center\">$user_stats[albums]</td>";
    print "<td align=\"center\">$user_stats[equipment]</td>";
    print "<td>".emit_date_html($viewer_user_data['member_since'], 'date')."</td>";
    print "</tr>";
  }

  $user_count = pg_num_rows($user_list);
  for ($i=0; $i < $user_count; $i++) {
    $user = pg_fetch_assoc($user_list, $i);
    if ($user['identifier'] != $viewer_user_data['identifier']) {
      $client = FALSE;
      if ($user['identifier'] != "" && $viewer_user_data['identifier'] != "") {
        $client = pg_fetch_assoc(pg_query ($database, "select identifier from client where users='$user[identifier]' and client='$viewer_user_data[identifier]' and status=2"));
      }

      if ($viewer_user_data['type'] == PO_USER_TYPE_ADMIN) {
	$user_stats = pg_fetch_assoc(pg_query($database, "
    select count_photos_by_user($user[identifier]) as photos,
           count_folders_by_user($user[identifier]) -1 as folders,
           count_albums_by_user($user[identifier]) - 1 as albums,
           count_equipment_by_user($user[identifier]) as equipment"));
      } else if ($client != FALSE) {
        $user_stats = pg_fetch_assoc(pg_query($database, "
    select count_photos_by_user($user[identifier], $access[protected]) as photos,
           count_folders_by_user($user[identifier], $access[protected]) as folders,
           count_albums_by_user($user[identifier], $access[protected]) as albums,
           count_equipment_by_user($user[identifier], $access[protected]) as equipment"));
      } else {
        $user_stats = pg_fetch_assoc(pg_query($database, "
    select count_photos_by_user($user[identifier], $access[public]) as photos,
           count_folders_by_user($user[identifier], $access[public]) as folders,
           count_albums_by_user($user[identifier], $access[public]) as albums,
           count_equipment_by_user($user[identifier], $access[public]) as equipment"));
      }

      /* Don't list users with no photos unless we're an admin! */
      if (($viewer_user_data['type'] < PO_USER_TYPE_ADMIN) &&
	  ($user_stats['photos'] == 0)) {
	continue;
      }

      /* Hide users that request it, unless we're an admin! */
      if (($viewer_user_data['type'] < PO_USER_TYPE_ADMIN) &&
	  $user['hide'] == 't') {
	continue;
      }

      print "<tr>";
      print "<td><a href=\"user.php?user=$user[identifier]\">$user[last_name], $user[first_name]</a></td>";
      print "<td align=\"center\">$user_stats[photos]</td>";
      print "<td align=\"center\">$user_stats[folders]</td>";
      print "<td align=\"center\">$user_stats[albums]</td>";
      print "<td align=\"center\">$user_stats[equipment]</td>";
      print "<td>".emit_date_html($user['member_since'], 'date')."</td>";
      print "</tr>";
    }
  }
  print "</table>\n";
  print "</div>";
}

function display_albums($database, $album_id, $owner_user_id, $owner_user_type, $display_header, $event = FALSE) {
  global $access;
  global $po_user;
  global $strings;
  global $passwords;

  $viewer_user_id = $po_user['id'];
  $viewer_user_type = $po_user['type'];
  $photos_in_spool = 0;

  if ($album_id == "") {
    $selector = "parent_album is null";
    $display_special_albums = TRUE;
  } else {
    $selector = "parent_album=$album_id";
    $display_special_albums = FALSE;
  }

  if ($event === FALSE)
    $all_albums = pg_query($database, "
     select album.identifier, caption, date_of_creation, access_rights, date_changed
       from album
      where users=$owner_user_id
        and $selector
        and can_access_album(album.identifier, $viewer_user_id, '{".$passwords."}')
      order by date_of_creation desc");
  else 
    $all_albums = pg_query($database, "
     select album.identifier, caption, date_of_creation, access_rights, date_changed
       from album
      where event = $event
        and can_access_album(album.identifier, $viewer_user_id, '{".$passwords."}')
      order by date_of_creation desc");

  $num_of_albums = pg_num_rows($all_albums);

  if (($viewer_user_id != $owner_user_id) && ($num_of_albums == 0))
    return;

  print "<div class=\"folder\">";
  print "<table class=\"folder\">\n";
  if ($display_header == "with_header") {
    print "<tr class=\"folder_top\">\n";
    if (($viewer_user_id == $owner_user_id) || ($viewer_user_type == PO_USER_TYPE_ADMIN))
      $colspan = 8;
    else
      $colspan = 6;
    print "<td colspan=\"$colspan\">".$strings['generic']['albums']."</td>\n";
    print "</tr>\n";
  }
  print "<tr>\n";
  print "<th width=\"46%\">".$strings['generic']['album_name']."</th>\n";
  print "<th width=\"6%\">".$strings['generic']['subalbums']."</th>\n";
  print "<th width=\"6%\">".$strings['generic']['photos']."</th>\n";
  print "<th width=\"13%\">".$strings['generic']['created']."</th>\n";
  print "<th width=\"13%\">".$strings['generic']['changed']."</th>\n";
  if ($viewer_user_id != FALSE) {
    print "<th width=\"6%\">".$strings['generic']['access']."</th>\n";
  }
  if ($viewer_user_id == $owner_user_id) {
    print "<th align=\"center\" colspan=\"2\" width=\"10%\">\n";
    print "<a href=\"album.add.php?parent=$album_id\">".$strings['generic']['add_album']."</a>";
    print "</th>\n";
  } else if ($viewer_user_type == PO_USER_TYPE_ADMIN) {
    print "<th colspan=\"2\" width=\"10%\"></th>";
  }
  print "</tr>\n";

  for ($i = 0; $i < $num_of_albums; $i++) {
    $album = pg_fetch_assoc($all_albums, $i);
    if (($album['caption'] == 'Spool' || 
	 $album['identifier'] == $po_user['spool_album']) &&
	$display_special_albums) {
      $photos_in_spool = photos_in_spool($database);
      $spool = $album;
    } else {
      $album_stats =  pg_fetch_assoc(pg_query($database, "select count_subalbums_by_album($album[identifier], $viewer_user_id, '{".$passwords."}') as albums, count_photos_by_album($album[identifier], $viewer_user_id, '{".$passwords."}') as photos"));
      
      print "<tr>\n";
      print "<td><a href=\"album.php?album=$album[identifier]\">$album[caption]</a></td>\n";
      print "<td>$album_stats[albums]</td>\n";
      print "<td>$album_stats[photos]</td>\n";
      
      print "<td>".emit_date_html($album['date_of_creation'], 'date'). "</td>";
      print "<td>".emit_date_html($album['date_changed'], 'date'). "</td>";
      if ($viewer_user_id != FALSE) {
	print emit_td($strings['generic'][$access[$album['access_rights']]]);
      }
      if (($viewer_user_id == $owner_user_id) || ($viewer_user_type == PO_USER_TYPE_ADMIN)) {
	print "<td width=\"5%\"><a href=\"album.edit.php?album=$album[identifier]\">".$strings['generic']['edit']."</a></td>\n";
	print "<td width=\"5%\"><a href=\"album.del.php?album=$album[identifier]\">".$strings['generic']['delete']."</a></td>\n";
      }
      print "</tr>\n";
    }
  }
  if ((($viewer_user_id == $owner_user_id) || 
       ($viewer_user_type == PO_USER_TYPE_ADMIN)) && 
      $display_special_albums && ($photos_in_spool > 0)) {
    print "<tr>\n";
    print "<td colspan=\"2\"><a href=\"album.php?album=$spool[identifier]\">$spool[caption]</a></td>\n";
    print "<td width=\"5%\">$photos_in_spool</td>\n";
    print "<td align=\"right\" colspan=\"5\">".$strings['generic']['print']." <a href=\"print.php?print_type=label\">".$strings['generic']['label']."</a> / <a href=\"print.php?print_type=brochure\">".$strings['generic']['brochure']."</a> | ".$strings['generic']['export']." <a href=\"export.php?export_type=photo\">".$strings['generic']['plain']."</a> / <a href=\"export.php?export_type=photo_xml\">XML</a> | <a href=\"my.tools.php?selector=1&amp;type=spool\">".$strings['generic']['bulk_update']."</a>  | <a href=\"spool.empty.php?user=$owner_user_id\">".$strings['generic']['empty']."</a></td>";
    print "</tr>\n";   
  }
  print "</table>\n";
  print "</div>\n";
}


function display_folders($database, $folder_id, $owner_user_id, $display_header, $event = FALSE) {
  global $access;
  global $po_user;
  global $strings;
  global $passwords;

  $viewer_user_id = $po_user['id'];
  $viewer_user_type = $po_user['type'];
  $photos_in_trash = 0;

  if ($folder_id == "") {
    $folder_selector = "parent_folder is null";
    $display_special_folders = TRUE;
  } else {
    $folder_selector = "parent_folder=$folder_id";
    $display_special_folders = FALSE;
  }

  if ($event === FALSE)
    $all_folders = pg_query($database, "select folder.identifier, caption, date_of_creation, access_rights, date_changed from folder where users = $owner_user_id and $folder_selector and can_access_folder(folder.identifier, $viewer_user_id, '{".$passwords."}') order by date_of_creation desc");
  else 
    $all_folders = pg_query($database, "select folder.identifier, caption, date_of_creation, access_rights, date_changed from folder where event = $event and can_access_folder(folder.identifier, $viewer_user_id, '{".$passwords."}') order by date_of_creation desc");

  $num_of_folders = pg_num_rows($all_folders);

  if (($viewer_user_id != $owner_user_id) && ($num_of_folders == 0))
    return;

  print "<div class=\"folder\">";
  print "<table class=\"folder\">\n";
  if ($display_header == "with_header") {
    print "<tr class=\"folder_top\">\n";
    if (($viewer_user_id == $owner_user_id) || ($viewer_user_type == PO_USER_TYPE_ADMIN))
      $colspan = 8;
    else
      $colspan = 6;
    if ($folder_id == "")
      print "<td colspan=\"$colspan\">".$strings['generic']['folders']."</td>\n";
    else 
      print "<td colspan=\"$colspan\">".$strings['generic']['subfolders']."</td>\n";
    print "</tr>\n";
  }
  print "<tr>\n";
  print "<th width=\"46%\">".$strings['generic']['folder_name']."</th>\n";
  print "<th width=\"6%\">".$strings['generic']['subfolders']."</th>\n";
  print "<th width=\"6%\">".$strings['generic']['photos']."</th>\n";
  print "<th width=\"13%\">".$strings['generic']['created']."</th>\n";
  print "<th width=\"13%\">".$strings['generic']['changed']."</th>\n";
  if ($viewer_user_id != FALSE) {
    print "<th width=\"6%\">".$strings['generic']['access']."</th>\n";
  }
  if ($viewer_user_id == $owner_user_id) {
    print "<th align=\"center\" colspan=\"2\" width=\"10%\">\n";
    print "<a href=\"folder.add.php?parent=$folder_id\">".$strings['generic']['add_folder']."</a>\n";
    print "</th>";
  } else if ($viewer_user_type == PO_USER_TYPE_ADMIN) {
    print "<th colspan=\"2\" width=\"10%\"></th>";
  }
  print "</tr>\n";

  /* trash is not a user folder */
  for ($i = 0; $i < $num_of_folders; $i++) {
    $folder = pg_fetch_assoc($all_folders, $i);
    if (($folder['caption'] == 'Trash' || $folder['identifier'] == $po_user['trash_folder']) && $display_special_folders) {
      $photos_in_trash = pg_fetch_assoc(pg_query($database, "select count_photos_by_folder($folder[identifier], $viewer_user_id, '{".$passwords."}') as photos"));
      $trash = $folder;
    } else {
      print "<tr>\n";
      print "<td><a href=\"folder.php?folder=$folder[identifier]\">$folder[caption]</a></td>\n";
      $folder_stats = pg_fetch_assoc(pg_query($database, "select count_subfolders_by_folder($folder[identifier], $viewer_user_id, '{".$passwords."}') as folders, count_photos_by_folder($folder[identifier], $viewer_user_id, '{".$passwords."}') as photos"));

      print "<td>$folder_stats[folders]</td>\n";
      print "<td>$folder_stats[photos]</td>\n";

      print "<td>".emit_date_html($folder['date_of_creation'], 'date'). "</td>";
      print "<td>".emit_date_html($folder['date_changed'], 'date'). "</td>";
      if ($viewer_user_id != FALSE) {
	print emit_td($strings['generic'][$access[$folder['access_rights']]]);
      }
      if (($viewer_user_id == $owner_user_id) || ($viewer_user_type == PO_USER_TYPE_ADMIN)) {
        print "<td width=\"5%\"><a href=\"folder.edit.php?folder=$folder[identifier]\">".$strings['generic']['edit']."</a></td>\n";
        print "<td width=\"5%\"><a href=\"folder.del.php?folder=$folder[identifier]\">".$strings['generic']['delete']."</a></td>\n";
      }
      print "</tr>\n";
    }
  }
  if ((($viewer_user_id == $owner_user_id) || ($viewer_user_type == PO_USER_TYPE_ADMIN)) && $display_special_folders) {
    if ($photos_in_trash["photos"] > 0) {
      print "<tr>\n";
      print "<td colspan=\"2\"><a href=\"folder.php?folder=$trash[identifier]\">".$strings['generic']['trash']."</a></td>\n";
      print "<td width=\"5%\">$photos_in_trash[photos]</td>\n";
      print "<td></td>";
      print "<td></td>";
      print "<td>".$strings['generic']['private']."</td>\n";
      print "<td align=\"center\" colspan=\"2\"><a href=\"trash.empty.php?user=$owner_user_id\">".$strings['generic']['empty_trash']."</a></td>\n";
      print "</tr>\n";
    }
  }
  print "</table>\n";
  print "</div>";
}

function display_folder_with_users($database, $my_user_id, $folder_list) {
  global $strings;

  print "<div class=\"folder\">";
  print "<table class=\"folder\">\n";

  print "<tr>\n";
  print "<th width=\"50%\">".$strings['generic']['folder']."</th>\n";
  print "<th width=\"5%\">".$strings['generic']['size']."</th>\n";
  print "<th width=\"5%\">".$strings['generic']['subfolders']."</th>\n";
  print "<th width=\"20%\">".$strings['generic']['created']."</th>\n";
  print "<th width=\"30%\">".$strings['generic']['user']."</th>\n";
  print "</tr>\n";

  $user_count = pg_num_rows($folder_list);
  for ($i=0; $i < $user_count; $i++) {
    $folder = pg_fetch_assoc($folder_list, $i);
    print "<tr>\n";
    print "<td><a href=\"folder.php?folder=$folder[folder_id]\">$folder[caption]</a></td>";
    print "<td>$folder[photos]</td>";
    print "<td>$folder[subs]</td>";
    print "<td>".emit_date_html($folder['date_of_creation'], 'date'). "</td>";
    print "<td><a href=\"user.php?user=$folder[user_id]\">$folder[first_name] $folder[last_name]</a></td>";
    print "</tr>\n";
  }
  print "</table>\n";
  print "</div>\n";
}

function display_album_with_users($database, $my_user_id, $album_list) {
  global $strings;

  print "<div class=\"folder\">";
  print "<table class=\"folder\">\n";

  print "<tr>\n";
  print "<th width=\"50%\">".$strings['generic']['album']."</th>\n";
  print "<th width=\"5%\">".$strings['generic']['size']."</th>\n";
  print "<th width=\"5%\">".$strings['generic']['subalbums']."</th>\n";
  print "<th width=\"20%\">".$strings['generic']['created']."</th>\n";
  print "<th width=\"30%\">".$strings['generic']['user']."</th>\n";
  print "</tr>\n";

  $user_count = pg_num_rows($album_list);
  for ($i=0; $i < $user_count; $i++) {
    $album = pg_fetch_assoc($album_list, $i);
    print "<tr>\n";
    print "<td><a href=\"album.php?album=$album[album_id]\">$album[caption]</a></td>";
    print "<td>$album[photos]</td>";
    print "<td>$album[subs]</td>";
    print "<td>".emit_date_html($album['date_of_creation'], 'date'). "</td>";
    print "<td><a href=\"user.php?user=$album[user_id]\">$album[first_name] $album[last_name]</a></td>";
    print "</tr>\n";
  }
  print "</table>\n";
  print "</div>\n";
}


function display_contact_info($database, $user_id) {
  global $strings;

  $user_info = pg_fetch_assoc(pg_query($database, "select first_name, last_name, company, contact, member_since from users where identifier=$user_id"));

  if (!$user_info)
    return;

  $contact_info = pg_fetch_assoc(pg_query($database, "select address1, address2, city, zipcode, state, country, phone, email, url
                                                      from view_contact_info
                                                     where identifier=$user_info[contact]"));

  print "<table class=\"listing\">\n";
  print "<tr class=\"folder_top\"><td colspan=\"2\">Contact Information</td></tr>\n";
  if ($user_info['first_name'] != "")
    print "<tr><td>".$strings['user']['first_name']."</td><td>$user_info[first_name]</td></tr>\n";
  if ($user_info['last_name'] != "")
    print "<tr><td>".$strings['user']['last_name']."</td><td>$user_info[last_name]</td></tr>\n";
  if ($user_info['company'] != "")
    print "<tr><td>".$strings['user']['company']."</td><td>$user_info[company]</td></tr>\n";
  if ($contact_info['address1'] != "" || $contact_info['address2'] != "")
    print "<tr><td>".$strings['user']['address']."</td><td>$contact_info[address1]<br/>$contact_info[address2]</td></tr>\n";
  if ($contact_info['city'] != "")
    print "<tr><td>".$strings['user']['city']."</td><td>$contact_info[city]</td></tr>\n";
  if ($contact_info['zipcode'] != "")
    print "<tr><td>".$strings['user']['post_code']."</td><td>$contact_info[zipcode]</td></tr>\n";
  if ($contact_info['state'] != "")
    print "<tr><td>".$strings['user']['state']."</td><td>$contact_info[state]</td></tr>\n";
  if ($contact_info['country'] != "")
    print "<tr><td>".$strings['user']['country']."</td><td>$contact_info[country]</td></tr>\n";
  if ($contact_info['email'] != "")
    print "<tr><td>".$strings['user']['email']."</td><td><a href=\"mailto:$contact_info[email]\">$contact_info[email]</a></td></tr>\n";
  if ($contact_info['url'] != "")
    print "<tr><td>".$strings['user']['web']."</td><td><a href=\"$contact_info[url]\">$contact_info[url]</a></td></tr>\n";
  if ($contact_info['phone'] != "")
    print "<tr><td>".$strings['user']['phone']."</td><td>$contact_info[phone]</td></tr>\n";
  if ($user_info['member_since'] != "")
    print "<tr><td>".$strings['generic']['member_since']."</td><td>".emit_date_html($user_info['member_since'], "date"). "</td></tr>\n";
  print "</table>\n";
}

function display_label_layout($left_margin, $top_margin, $x_size, $y_size, $x_space, $y_space, $columns, $rows, $show_leading_blanks) {
  global $strings;

  print "<table align=\"center\" border=\"0\" cellspacing=\"0\" cellpadding=\"0\">\n";
  print "<tr>\n";
  print "<td align=\"center\"  colspan=\"2\">";
  if ($show_leading_blanks)
    print "Leading blanks";
  print "</td>\n";
  print "<td align=\"center\" >".$strings['label']['margin']."</td>\n";
  print "<td align=\"center\" >".$strings['label']['width']."</td>\n";
  print "<td align=\"center\" >".$strings['label']['space']."</td>\n";
  print "<td colspan=\"2\"></td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<td colspan=\"7\"></td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<td align=\"center\"  colspan=\"2\">";
  if ($show_leading_blanks)
    print "<input type=\"text\" name=\"leading_blanks\" value=\"0\" size=\"6\"/>";
  print "</td>\n";
  print "<td align=\"center\" ><input type=\"text\" name=\"left_margin\" value=\"$left_margin\" size=\"6\"/></td>\n";
  print "<td align=\"center\" ><input type=\"text\" name=\"x_size\" value=\"$x_size\" size=\"6\"/></td>\n";
  print "<td align=\"center\" ><input type=\"text\" name=\"x_space\" value=\"$x_space\" size=\"6\"/></td>\n";
  print "<td colspan=\"2\"></td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<td colspan=\"7\"></td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<td></td>\n";
  print "<td><img src=\"themes/print.label.11.gif\" alt=\"\" /></td>\n";
  print "<td><img src=\"themes/print.label.12.gif\" alt=\"\" /></td>\n";
  print "<td><img src=\"themes/print.label.13.gif\" alt=\"\" /></td>\n";
  print "<td><img src=\"themes/print.label.14.gif\" alt=\"\" /></td>\n";
  print "<td><img src=\"themes/print.label.15.gif\" alt=\"\" /></td>\n";
  print "<td></td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<td align=\"right\" >".$strings['label']['margin']."&nbsp;<input type=\"text\" name=\"top_margin\" value=\"$top_margin\" size=\"6\"/>&nbsp;</td>\n";
  print "<td colspan=\"5\"><img src=\"themes/print.label.21.gif\" alt=\"\" /></td>\n";
  print "<td></td>";
  print "</tr>\n";

  print "<tr>\n";
  print "<td align=\"right\" >".$strings['label']['height']."&nbsp;<input type=\"text\" name=\"y_size\" value=\"$y_size\" size=\"6\"/>&nbsp;</td>\n";
  print "<td colspan=\"5\"><img src=\"themes/print.label.31.gif\" alt=\"\" /></td>\n";
  print "<td align=\"center\" >&nbsp;";
  print "<select name=\"rows\">\n";
  for ($i=1; $i < 51; $i++) {
    print emit_option($i, $i, $i == $rows);
  }
  print "</select>&nbsp;".$strings['label']['rows']."\n";
  print "</td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<td align=\"right\" >".$strings['label']['space']."&nbsp;<input type=\"text\" name=\"y_space\" value=\"$y_space\" size=\"6\"/>&nbsp;</td>\n";
  print "<td colspan=\"5\"><img src=\"themes/print.label.41.gif\" alt=\"\" /></td>\n";
  print "<td></td>";
  print "</tr>\n";

  print "<tr>\n";
  print "<td></td>\n";
  print "<td colspan=\"5\"><img src=\"themes/print.label.51.gif\" alt=\"\" /></td>\n";
  print "<td></td>";
  print "</tr>\n";

  print "<tr>\n";
  print "<td colspan=\"7\"></td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<td colspan=\"3\"></td>\n";
  print "<td align=\"center\" >";
  print "&nbsp;<br/>\n";
  print "<select name=\"columns\">\n";
  for ($i=1; $i < 11; $i++) {
    print emit_option($i, $i, $i == $columns);
  }
  print "</select>\n";
  print "</td>\n";
  print "<td colspan=\"4\"></td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<td colspan=\"7\"></td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<td colspan=\"3\"></td>\n";
  print "<td align=\"center\" >".$strings['label']['columns']."</td>\n";
  print "<td colspan=\"3\"></td>\n";
  print "</tr>\n";

  print "</table>\n";
}

function makeZanza($string, $default) {
  if ($string) {
    if (strlen($string) > 40) {
      $zanza = substr($string, 0, 40);
      $zanza = $zanza . "...";
    } else {
      $zanza = $string;
    }
  } else {
    $zanza = $default;
  }
  return $zanza;
}

function grant_access_to_original($is_original_present, $is_mine, $is_admin, $is_original_public) {
  if (!$is_original_present)
    return false;

  if ($is_mine)
    return true;

  if ($is_admin)
    return true;

  if ($is_original_public)
    return true;

  return false;
}

// See http://sylvana.net/jpegcrop/exif_orientation.html for an explanation of these numbers

function map_transform($orientation, $transform) {
	switch ($transform) {
		case 'r90':
			switch ($orientation) {
				case 1:
					return 8;
				case 2:
					return 7;
				case 3:
					return 6;
				case 4:
					return 5;
				case 5:
					return 2;
				case 6:
					return 1;
				case 7:
					return 4;
				case 8:
					return 3;
				default:
			}
			break;
		case 'r180':
			switch ($orientation) {
				case 1:
					return 3;
				case 2:
					return 4;
				case 3:
					return 1;
				case 4:
					return 2;
				case 5:
					return 7;
				case 6:
					return 8;
				case 7:
					return 5;
				case 8:
					return 6;
				default:
			}
			break;
		case 'r270':
			switch ($orientation) {
				case 1:
					return 6;
				case 2:
					return 5;
				case 3:
					return 8;
				case 4:
					return 7;
				case 5:
					return 4;
				case 6:
					return 3;
				case 7:
					return 2;
				case 8:
					return 1;
				default:
			}
			break;
		case 'flip':
			switch ($orientation) {
				case 1:
					return 4;
				case 2:
					return 3;
				case 3:
					return 2;
				case 4:
					return 1;
				case 5:
					return 6;
				case 6:
					return 5;
				case 7:
					return 8;
				case 8:
					return 7;
				default:
			}
			break;
		case 'flop':
			switch ($orientation) {
				case 1:
					return 2;
				case 2:
					return 1;
				case 3:
					return 4;
				case 4:
					return 3;
				case 5:
					return 8;
				case 6:
					return 7;
				case 7:
					return 6;
				case 8:
					return 5;
				default:
			}
			break;
		default:
        }

	return $orientation;
}


/* Add a folder to the database */
function add_folder($database, $userid, $parent, $access_rights, $name, $descr, $password, $event)
{
  global $strings;

  $name = pg_escape_string($name);
  $descr = pg_escape_string($descr);
  $password = pg_escape_string($password);
  $access_rights = pg_escape_string($access_rights);
  $event = pg_escape_string($event);

  if ($event != "null") $client = "'$event'";

  if (($name == 'Trash') || ($name == '')) {
    site_push_error($strings['errors']['illegal_name']);
    return FALSE;
  } else {
    $string = "insert into folder (identifier, parent_folder, users, caption, description, access_rights, password, event)
                                 values (nextval('folder_id_sequence'), $parent, 
$userid, '$name', '$descr', $access_rights, '$password', $event)";

    pg_query($database, "begin");
    $result = pg_query($database, $string);
    if ($result) {
      pg_query($database, "commit");
      return TRUE;
    } else {
      site_push_error($strings['errors']['db_insert_failed']);
      return FALSE;
    }
  }
}

function photos_between_dates($database, $userid, $from, $to)
{
  $from = date("Y-m-d H:i", $from);
  $to = date("Y-m-d H:i", $to);

  $result = pg_fetch_row(pg_query($database, "SELECT count(identifier) FROM photo WHERE users = $userid AND date_of_exposure BETWEEN '$from' AND '$to'"));
  return $result[0];
}

function display_iptc_form($database, $bulk_update, 
			   $title, $author, $keywords, $headline, 
			   $caption_writer, $caption, $category,
			   $supp_category, $credit, $copyright,
			   $web_statement, $instructions, $source, 
			   $trans_ref, $exp_date, $location, 
			   $access, $hide_original, $store_url, $comments) {
  global $strings;
  global $po_user;

  /* Retrieve location info */
  if ($po_user['type'] != PO_USER_TYPE_ADMIN)
    $location_mod = " where private is false or users=$po_user[id] ";
  else
    $location_mod = "";

  $users_location  = pg_query($database, "select identifier, country, state, city, place from view_location $location_mod order by country, state, city, place");

  /* And now for the form */

  print "<tr>"; 
  if ($bulk_update)
    print emit_td($strings['generic']['title'] . "<input type=\"checkbox\" name=\"to_update[]\" value=\"set_title\"/>"); 
  else
    print emit_td($strings['generic']['title']); 
  print emit_td("<textarea name=\"title\" rows=\"2\" cols=\"60\">$title</textarea>");
  print "</tr>\n";

  print "<tr>"; 
  if ($bulk_update)
    print emit_td($strings['generic']['author'] . "<input type=\"checkbox\" name=\"to_update[]\" value=\"set_author\"/>"); 
  else
    print emit_td($strings['generic']['author']); 
  print emit_td("<textarea name=\"author\" rows=\"2\" cols=\"60\">$author</textarea>");
  print "</tr>\n";

  if ($bulk_update) {
    print "<tr>"; 
    print emit_td($strings['generic']['set_keywords'] ."<input type=\"checkbox\" name=\"to_update[]\" value=\"set_keyword\"/>"); 
    print emit_td("<textarea name=\"set_keywords\" title=\"".$strings['import']['keywords']."\" rows=\"2\" cols=\"60\"></textarea>");
    print "</tr>\n";

    print "<tr>"; 
    print emit_td($strings['generic']['clear_keywords']."<input type=\"checkbox\" name=\"to_update[]\" value=\"clear_keyword\"/>"); 
    print emit_td("<textarea name=\"clear_keywords\" title=\"".$strings['import']['keywords']."\" rows=\"2\" cols=\"60\"></textarea>");
    print "</tr>\n";
  } else {
    print "<tr>"; 
    print emit_td($strings['generic']['keywords']); 
    print emit_td("<textarea name=\"keywords\" title=\"".$strings['import']['keywords']."\" rows=\"2\" cols=\"60\">$keywords</textarea>");
    print "</tr>\n";
  }
  
  print "<tr>"; 
  if ($bulk_update) {
    print emit_td($strings['generic']['headline']. "<input type=\"checkbox\" name=\"to_update[]\" value=\"set_headline\"/>"); 
  } else {
    print emit_td($strings['generic']['headline']);
  }
  print emit_td("<textarea name=\"headline\" rows=\"2\" cols=\"60\">$headline</textarea>");
  print "</tr>\n";
  
  print "<tr>"; 
  if ($bulk_update) 
    print emit_td($strings['generic']['caption_writer'] . "<input type=\"checkbox\" name=\"to_update[]\" value=\"set_caption_writer\"/>"); 
  else
    print emit_td($strings['generic']['caption_writer']); 
  print emit_td("<textarea name=\"caption_writer\" rows=\"2\" cols=\"60\">$caption_writer</textarea>");
  print "</tr>\n";

  print "<tr>"; 
  if ($bulk_update) 
    print emit_td($strings['generic']['caption'] ."<input type=\"checkbox\" name=\"to_update[]\" value=\"set_caption\"/>"); 
  else
    print emit_td($strings['generic']['caption']); 
  print emit_td("<textarea name=\"caption\" rows=\"2\" cols=\"60\">$caption</textarea>");
  print "</tr>\n";
  
  print "<tr>"; 
  if ($bulk_update) 
    print emit_td($strings['generic']['category']."<input type=\"checkbox\" name=\"to_update[]\" value=\"set_category\"/>"); 
  else
    print emit_td($strings['generic']['category']); 

  print emit_td("<textarea name=\"category\" title=\"Category\" rows=\"2\" cols=\"60\">$category</textarea>");
  print "</tr>\n";

  print "<tr>"; 
  if ($bulk_update) 
    print emit_td($strings['generic']['supp_category']. "<input type=\"checkbox\" name=\"to_update[]\" value=\"set_supplemental_category\"/>"); 
  else
    print emit_td($strings['generic']['supp_category']); 
  print emit_td("<textarea name=\"supplemental_category\" rows=\"2\" cols=\"60\">$supp_category</textarea>");
  print "</tr>\n";
  
  print "<tr>"; 
  if ($bulk_update)
    print emit_td($strings['generic']['credit']."<input type=\"checkbox\" name=\"to_update[]\" value=\"set_credit\"/>"); 
  else 
    print emit_td($strings['generic']['credit']); 
  print emit_td("<textarea name=\"credit\" rows=\"2\" cols=\"60\">$credit</textarea>");
  print "</tr>\n";
  
  print "<tr>"; 
  if ($bulk_update)
    print emit_td($strings['generic']['copyright']. "<input type=\"checkbox\" name=\"to_update[]\" value=\"set_copyright\"/>"); 
  else
    print emit_td($strings['generic']['copyright']); 
  print emit_td("<textarea name=\"copyright\" rows=\"2\" cols=\"60\">$copyright</textarea>");
  print "</tr>\n";
  
  print "<tr>"; 
  if ($bulk_update)
    print emit_td($strings['generic']['web_statement']."<input type=\"checkbox\" name=\"to_update[]\" value=\"set_web_statement\"/>"); 
  else 
    print emit_td($strings['generic']['web_statement']); 
  print emit_td("<textarea name=\"web_statement\" rows=\"2\" cols=\"60\">$web_statement</textarea>");
  print "</tr>\n";
  
  print "<tr>"; 
  if ($bulk_update)
    print emit_td($strings['generic']['instructions']."<input type=\"checkbox\" name=\"to_update[]\" value=\"set_instructions\"/>"); 
  else
    print emit_td($strings['generic']['instructions']); 
  print emit_td("<textarea name=\"instructions\" rows=\"2\" cols=\"60\">$instructions</textarea>");
  print "</tr>\n";

  print "<tr>"; 
  if ($bulk_update)
    print emit_td($strings['generic']['source']."<input type=\"checkbox\" name=\"to_update[]\" value=\"set_source\"/>"); 
  else
    print emit_td($strings['generic']['source']); 
  print emit_td("<textarea name=\"source\" rows=\"2\" cols=\"60\">$source</textarea>");
  print "</tr>\n";

  print "<tr>"; 
  if ($bulk_update)
    print emit_td($strings['generic']['trans_ref']."<input type=\"checkbox\" name=\"to_update[]\" value=\"set_transmission_reference\"/>"); 
  else
    print emit_td($strings['generic']['trans_ref']); 
  print emit_td("<textarea name=\"transmission_reference\" rows=\"2\" cols=\"60\">$trans_ref</textarea>");
  print "</tr>\n";

  print "<tr>"; 
  if ($bulk_update)
    print emit_td($strings['generic']['exposed']."<input type=\"checkbox\" name=\"to_update[]\" value=\"set_date_of_exposure\"/>");
  else
    print emit_td($strings['generic']['exposed']);
  print emit_td(emit_date_input_form("exposure_timestamp", $exp_date));
  print "</tr>\n";

  print "<tr>";
  if ($bulk_update) 
    print emit_td($strings['generic']['location']."<input type=\"checkbox\" name=\"to_update[]\" value=\"set_location\"/>");
  else
    print emit_td($strings['generic']['location']);
  print emit_td(emit_combo("location", $users_location, $location, ", ", 1));
  print "</tr>\n";

  print "<tr>";
  if ($bulk_update)
    print emit_td($strings['generic']['access']."<input type=\"checkbox\" name=\"to_update[]\" value=\"set_access_rights\"/>");
  else
    print emit_td($strings['generic']['access']);
  print "<td>";
  display_access_combo("access_rights", $access, "");
  print "</td></tr>\n";

  print "<tr>";
  if ($bulk_update)
    print emit_td($strings['generic']['high_res_orig'] ."<input type=\"checkbox\" name=\"to_update[]\" value=\"set_hide_original\"/>");
  else
    print emit_td($strings['generic']['high_res_orig']);
  print "<td>";
  display_hide_original_combo("hide_original", $hide_original);
  print "</td></tr>\n";

  print "<tr>";
  if ($bulk_update)
    print emit_td($strings['photo']['store'] ."<input type=\"checkbox\" name=\"to_update[]\" value=\"set_store_url\"/>");
  else
    print emit_td($strings['photo']['store']);
  print emit_td("<textarea name=\"store_url\" rows=\"2\" cols=\"60\">$store_url</textarea>");
  print "</tr>\n";

  print "<tr>"; 
  if ($bulk_update)
    print emit_td($strings['generic']['remark'] . "<input type=\"checkbox\" name=\"to_update[]\" value=\"set_remark\"/>"); 
  else
    print emit_td($strings['generic']['remark']); 
  print emit_td("<textarea name=\"remark\" rows=\"2\" cols=\"60\">$comments</textarea>");
  print "</tr>\n";

}

function display_photo_tech_form($database, $bulk_update, 
				 $camera, $metering, $program, $lens,
				 $focal_length, $filter, $film, $iso,
				 $aperture, $shutter, $exp_comp, $ev_diff,
				 $flash, $flash_mode, $flash_comp, $scanner,
				 $scan_res, $scan_bit, $scan_multi, $support,
				 $latitude, $longitude, $altitude, $direction) {
  global $strings;
  global $po_user;

  /* First we need to retrieve all sorts of crap from the database */
  $users_camera    = pg_query($database, "select identifier, name, model, variation from view_camera where users=$po_user[id]");
  $users_lens      = pg_query($database, "select identifier, name, model, variation, s_min_aperture, s_max_aperture, s_min_focal_length, s_max_focal_length from view_lens where users=$po_user[id]");
  $users_film      = pg_query($database, "select identifier, name, model, iso       from view_film where users=$po_user[id]");
  $users_flash     = pg_query($database, "select identifier, name, model, variation from view_flash where users=$po_user[id]");
  $users_scanner   = pg_query($database, "select identifier, name, model, variation from view_scanner where users=$po_user[id]");
  $users_filter    = pg_query($database, "select identifier, name, model, variation from view_filter where users=$po_user[id]");
  $users_support   = pg_query($database, "select identifier, name, model, variation from view_support where users=$po_user[id]");
  
  $apertures       = pg_query($database, "select identifier, value from aperture order by value");
  $focal_lengths   = pg_query($database, "select identifier, value from focal_length order by value");
  $shutters        = pg_query($database, "select identifier, value from shutter");
  $ev_comps        = pg_query($database, "select identifier, value from ev_comp");
  $camera_metering = pg_query($database, "select identifier, value from camera_metering");
  $camera_program  = pg_query($database, "select identifier, value from camera_program");
  $flash_modes     = pg_query($database, "select identifier, value from flash_mode");
  $scan_resolutions= pg_query($database, "select identifier, value from scan_resolution");
  $scan_bitdepths  = pg_query($database, "select identifier, value from scan_bitdepth");
  $scan_multiscans = pg_query($database, "select identifier, value from scan_multiscan");
  $iso_overrides   = pg_query($database, "select identifier, value from film_iso order by value");

  /* Now for the actual form */
  print "<tr>";
  if ($bulk_update) 
    print emit_td($strings['generic']['camera'] ."<input type=\"checkbox\" name=\"to_update[]\" value=\"set_camera\"/>");
  else
    print emit_td($strings['generic']['camera']);
  print emit_td(emit_combo("camera", $users_camera, $camera, "", 0));
  print "</tr>\n";

  print "<tr>";
  if ($bulk_update)
    print emit_td($strings['generic']['camera_metering']."<input type=\"checkbox\" name=\"to_update[]\" value=\"set_camera_metering\"/>");
  else
    print emit_td($strings['generic']['camera_metering']);
  print emit_td(emit_combo("camera_metering", $camera_metering, $metering, "", 0));
  print "</tr>\n";

  print "<tr>";
  if ($bulk_update)
    print emit_td($strings['generic']['camera_program']."<input type=\"checkbox\" name=\"to_update[]\" value=\"set_camera_program\"/>");
  else
    print emit_td($strings['generic']['camera_program']);
  print emit_td(emit_combo("camera_program", $camera_program, $program, "", 0));
  print "</tr>\n";

  print "<tr>";
  if ($bulk_update)
    print emit_td($strings['generic']['lens']."<input type=\"checkbox\" name=\"to_update[]\" value=\"set_lens\"/>");
  else
    print emit_td($strings['generic']['lens']);
  print "<td>";
  display_lens_combo("lens", $users_lens, $lens, 0, 0);
  print "</td>";
  print "</tr>\n";

  print "<tr>";
  if ($bulk_update)
    print emit_td($strings['generic']['focal_length']."<input type=\"checkbox\" name=\"to_update[]\" value=\"set_focal_length\"/>");
  else
    print emit_td($strings['generic']['focal_length']);
  print emit_td(emit_combo("focal_length", $focal_lengths, $focal_length, "", 0));
  print "</tr>\n";

  print "<tr>";
  if ($bulk_update)
    print emit_td($strings['generic']['filter']."<input type=\"checkbox\" name=\"to_update[]\" value=\"set_filter\"/>");
  else
    print emit_td($strings['generic']['filter']);
  print emit_td(emit_combo("filter", $users_filter, $filter, "", 0));
  print "</tr>\n";

  print "<tr>";
  if ($bulk_update)
    print emit_td($strings['generic']['film']."<input type=\"checkbox\" name=\"to_update[]\" value=\"set_film\"/>");
  else
    print emit_td($strings['generic']['film']);
  print emit_td(emit_combo("film", $users_film, $film, "", 0));
  print "</tr>\n";

  print "<tr>";
  if ($bulk_update)
    print emit_td($strings['generic']['iso_override']."<input type=\"checkbox\" name=\"to_update[]\" value=\"set_iso_override\"/>");
  else
    print emit_td($strings['generic']['iso_override']);
  print emit_td(emit_combo("iso_override", $iso_overrides, $iso, "", 0));
  print "</tr>\n";

  print "<tr>";
  if ($bulk_update)
    print emit_td($strings['generic']['aperture']."<input type=\"checkbox\" name=\"to_update[]\" value=\"set_aperture\"/>");
  else
    print emit_td($strings['generic']['aperture']);
  print emit_td(emit_combo("aperture", $apertures, $aperture, "", 0));
  print "</tr>\n";

  print "<tr>";
  if ($bulk_update)
    print emit_td($strings['generic']['shutter_speed']."<input type=\"checkbox\" name=\"to_update[]\" value=\"set_shutter\"/>");
  else
    print emit_td($strings['generic']['shutter_speed']);
  print emit_td(emit_combo("shutter", $shutters, $shutter, "", 0));
  print "</tr>\n";

  print "<tr>";
  if ($bulk_update)
    print emit_td($strings['generic']['exp_comp']."<input type=\"checkbox\" name=\"to_update[]\" value=\"set_exp_comp\"/>");
  else
    print emit_td($strings['generic']['exp_comp']);
  print emit_td(emit_combo("exp_comp", $ev_comps, $exp_comp, "", 0));
  print "</tr>\n";

  print "<tr>";
  if ($bulk_update)
    print emit_td($strings['generic']['exp_diff']."<input type=\"checkbox\" name=\"to_update[]\" value=\"set_exp_diff\"/>");
  else
    print emit_td($strings['generic']['exp_diff']);
  print emit_td(emit_combo("exp_diff", $ev_comps, $ev_diff, "", 0));
  print "</tr>\n";

  print "<tr>";
  if ($bulk_update)
    print emit_td($strings['generic']['flash']."<input type=\"checkbox\" name=\"to_update[]\" value=\"set_flash\"/>");
  else
    print emit_td($strings['generic']['flash']);
  print emit_td(emit_combo("flash", $users_flash, $flash, "", 0));
  print "</tr>\n";

  print "<tr>";
  if ($bulk_update)
    print emit_td($strings['generic']['flash_mode']."<input type=\"checkbox\" name=\"to_update[]\" value=\"set_flash_mode\"/>");
  else
    print emit_td($strings['generic']['flash_mode']);
  print emit_td(emit_combo("flash_mode", $flash_modes, $flash_mode, "", 0));
  print "</tr>\n";

  print "<tr>";
  if ($bulk_update)
    print emit_td($strings['generic']['flash_comp']."<input type=\"checkbox\" name=\"to_update[]\" value=\"set_flash_comp\"/>");
  else
    print emit_td($strings['generic']['flash_comp']);
  print emit_td(emit_combo("flash_comp", $ev_comps, $flash_comp, "", 0));
  print "</tr>\n";

  print "<tr>";
  if ($bulk_update)
    print emit_td($strings['generic']['support']."<input type=\"checkbox\" name=\"to_update[]\" value=\"set_support\"/>");
  else
    print emit_td($strings['generic']['support']);
  print emit_td(emit_combo("support", $users_support, $support, "", 0));
  print "</tr>\n";

  print "<tr>";
  if ($bulk_update)
    print emit_td($strings['photo']['geo-location']. " <input type=\"checkbox\" name=\"to_update[]\" value=\"set_geo_location\"/>");
  else 
    print emit_td($strings['photo']['geo-location']);
  
  $latitude = htmlentities(latitude_dms($latitude, "latitude"));
  $longitude = htmlentities(latitude_dms($longitude, "longitude"));
  print emit_td($strings['photo']['latitude'].": <input type=\"text\" name=\"latitude\" value=\"$latitude\" title=\"eg '-14.221024'\" /><br/>" .
		$strings['photo']['longitude'].": <input type=\"text\" name=\"longitude\" value=\"$longitude\" title=\"eg '22.5123'\" /><br/>" .
		$strings['photo']['altitude'].": <input type=\"text\" name=\"altitude\" value=\"$altitude\" title=\"eg '25'\" /><br/>" .
		$strings['photo']['direction'].": <input type=\"text\" name=\"direction\" value=\"$direction\" title=\"eg '170.51'\" />");
  print "</tr>\n";

  print "<tr>";
  if ($bulk_update)
    print emit_td($strings['generic']['scanner']."<input type=\"checkbox\" name=\"to_update[]\" value=\"set_scanner\"/>");
  else
    print emit_td($strings['generic']['scanner']);
  print emit_td(emit_combo("scanner", $users_scanner, $scanner, "", 0));
  print "</tr>\n";

  print "<tr>";
  if ($bulk_update)
    print emit_td($strings['generic']['scan_parameters']."<input type=\"checkbox\" name=\"to_update[]\" value=\"set_scan_params\"/>");
  else
    print emit_td($strings['generic']['scan_parameters']);
  
  print "<td>";
  print emit_combo("scan_resolution", $scan_resolutions, $scan_res, "", 0);
  print $strings['generic']['dpi'] ." &nbsp;";
  print emit_combo("scan_bitdepth", $scan_bitdepths, 0, $scan_bit, 0);
  print $strings['generic']['bits'] ." &nbsp;";
  print emit_combo("scan_multiscan", $scan_multiscans, 0, $scan_multi, 0);
  print $strings['generic']['passes'] ." &nbsp;";
  print "</td></tr>\n";
}

function semi_string_to_array($str){
   $expr="/;(?=(?:[^\"]*\"[^\"]*\")*(?![^\"]*\"))/";
   $results=preg_split($expr,trim($str));
   return preg_replace("/^\"(.*)\"$/","$1",$results);
}
function space_string_to_array($str){
   $expr="/ (?=(?:[^\"]*\"[^\"]*\")*(?![^\"]*\"))/";
   $results=preg_split($expr,trim($str));
   return preg_replace("/^\"(.*)\"$/","$1",$results);
}

function po_get_image_size($file) {
  global $sys_exiftool;

  if (!is_executable($sys_exiftool)) 
    return getimagesize($file);

  $width = FALSE;
  $height = FALSE;

  $handle = popen("$sys_exiftool -ImageWidth -ImageHeight -Rows -Columns $file", "r");
  if ($handle) {
    while (!feof($handle)) {
      $line = fgets($handle);
      $parts = explode(":", $line, 2);
      $parts[0] = trim($parts[0]);
      if (!isset($parts[1])) continue;
      $parts[1] = trim($parts[1]);
      if ($parts[0] == "Image Width") $width = $parts[1];
      if ($parts[0] == "Image Height") $height = $parts[1];
      if ($parts[0] == "Rows") $height = $parts[1];
      if ($parts[0] == "Columns") $width = $parts[1];
      // print "<!-- XXX $parts[0] : $parts[1] -- $width $height -->\n";
    }
    pclose($handle);
  } else {
    // print "<!-- it broke.. -->\n";
  }

  if (!$width || !$height)
    return getimagesize($file);
  
  return array($width,$height);
}

function store_user_pref($dbh, $userid, $key, $value) {
  global $po_options_default;
  global $strings;

  $key2 = pg_escape_string($key);
  $value2 = pg_escape_string($value);

  pg_query($dbh, "begin");
  $result = pg_query($dbh, "DELETE FROM user_preferences WHERE owner = $userid AND key = '$key2'");
  if ($result && ($po_options_default[$key] != $value)) {
    $result = pg_query($dbh, "INSERT INTO user_preferences (owner, key, value) VALUES($userid, '$key2', '$value2')");
  }

  if ($result) {
    pg_query($dbh, "commit");
    return TRUE;
  } else {
    site_push_error($strings['errors']['db_insert_failed']);
    return FALSE;
  }
}

function refresh_user_prefs($database) {
  global $po_options;
  global $po_user;

  if (!$po_user['id']) {
    $po_user['id'] = 0;
    return;
  }

  $user_data = pg_fetch_row(pg_query($database, "select username, password, first_name, last_name, company, contact, type from users where identifier='$po_user[id]'"));

  $po_user['username']        = $user_data[0];
  // ignore password...
  $po_user['first_name']      = $user_data[2];
  $po_user['last_name']       = $user_data[3];
  $po_user['company']         = $user_data[4];

  $po_user['contact_id']      = $user_data[5];
  $po_user['type']            = $user_data[6];

  /* Look up spool album id */
  $users_spool = pg_query($database, "select album.identifier from album where caption = 'Spool' and users = $po_user[id]");
  $spool_album = pg_fetch_row($users_spool);
  $po_user['spool_album']     = $spool_album[0];

  /* Look up trash folder IDs */
  $query = pg_query($database, "select folder.identifier from folder where caption = 'Trash' and users = $po_user[id]");
  $folder = pg_fetch_row($query);
  $po_user['trash_folder']    = $folder[0];

  /* Contact information */
  $contact_info = pg_fetch_row(pg_query($database, "select address1, address2, city, zipcode, state, country, phone, email, url from contact_info where identifier='$po_user[contact_id]'"));

  $po_user['address1']        = $contact_info[0];
  $po_user['address2']        = $contact_info[1];
  $po_user['city']            = $contact_info[2];
  $po_user['zipcode']         = $contact_info[3];
  $po_user['state']           = $contact_info[4];
  $po_user['country']         = $contact_info[5];
  $po_user['phone']           = $contact_info[6];
  $po_user['email']           = $contact_info[7];
  $po_user['url']             = $contact_info[8];

  /* Slurp up all user preferences */
  $user_prefs = pg_query($database, "SELECT key, value FROM user_preferences WHERE owner = $po_user[id]");

  if ($user_prefs) {
    while (($row = pg_fetch_assoc($user_prefs)) !== FALSE) {
      /* We want to skip a couple of prefs */
      if ($row['key'] == "lang") continue;

      if ($row['key'] == 'passwords')
	$po_options[$row['key']] = unserialize($row['value']);
      else
	$po_options[$row['key']] = $row['value'];
    }
  }
 
  /* Update Session state */
  $po_user['last_updated'] = time();
  store_user_pref($database, $po_user['id'], 'last_updated', $po_user['last_updated']);
}

function emit_language_combo($lang) {
  global $strings;
  global $po_options;

  $rval = "";
  $i = 0;

  $rval .= "<tr>";
  $rval .= emit_td($strings['generic']['language']);
  $rval .= "<td><select name=\"lang\" onchange=\"location = 'login.php?lang=' + this.options[this.selectedIndex].value;\">";

  if ($dh = opendir("lang")) {
       while (($file = readdir($dh)) !== false) {
	 if ($file == "." || $file == "..") continue;
	 if ($file == "." || $file == "..") continue;
	 if ($file == "." || $file == "..") continue;
	 if (strchr($file, "~") !== false) continue;
	 if (strchr($file, ".php") === false) continue;

	 $file = substr($file, 0, 5);
	 include "lang/".$file.".php";
	 $rval .= emit_option($file, $strings['formats']['name'], $lang == $file);
       }
       closedir($dh);
   }  

  include "lang/".$po_options['lang'].".php";

  $rval .= "</select></td>";
  $rval .= "</tr>";

  return $rval;
}

function parse_latitude($value) {
  // my ($d, $m, $s) = ($val =~ /((?:[+-]?)(?=\d|\.\d)\d*(?:\.\d*)?)/g);

  if ($value == "")
    return "";
  
  $regexp = "/((?:[+-]?)(?=\d|\.\d)\d*(?:\.\d*)?)/";
  $res = array();
  $rval = "";

  if (preg_match_all($regexp, $value, $res)) {
    if (!isset($res[0][0])) $res[0][0] = 0;
    if (!isset($res[0][1])) $res[0][1] = 0;
    if (!isset($res[0][2])) $res[0][2] = 0;

    $rval = $res[0][0] + (($res[0][1] + ($res[0][2]/60))/60) ;
  }

  if (preg_match("/[^A-Z](S|W)$/i", $value))
    $rval = - $rval;
  
  return $rval;
}

function latitude_dms($latitude_deg, $type) {
  global $strings;

  if ($latitude_deg == "")
    return "";

  if ($type == "latitude") {
    if ($latitude_deg > 0) 
      $dir = $strings['generic']['north'];
    else 
      $dir = $strings['generic']['south'];
  } else {
    if ($latitude_deg > 0) 
      $dir = $strings['generic']['east'];
    else 
      $dir = $strings['generic']['west'];
  }
  
  if ($latitude_deg < 0)
    $latitide_deg = - $latitude_deg;

  $deg = intval($latitude_deg);
  $min = intval(($latitude_deg-$deg)*60);
  $sec = ($latitude_deg-$deg-($min/60))*3600;

  return sprintf("%d deg %02d' %02.02f\" $dir", $deg, $min, $sec, $dir);
}

function user_disp_string($owner_id, $user_id, $first, $last) {
  global $strings;
  
  if (!$owner_id) {
    return (emit_a("user.php", $strings['generic']['users']));
  } else {
    if ($owner_id == $user_id) {
      $str = $strings['users']['my_folders'];
    } else {
      $str = sprintf("%s, %s", $first, $last);
    }
    return (emit_a("user.php?user=$owner_id", $str));
  }
}

function generate_html_users_combo($identifier, $database, $default_user_id, $all = FALSE, $none = FALSE) {
  global $strings;
  global $po_user;

  $user_list = pg_query($database, "select identifier, last_name, first_name, hide from users where users.type > ".PO_USER_TYPE_CLIENT." order by last_name, first_name");

  $html_string = "<select name=\"$identifier\" title=\"".$strings['search']['search_user'] ."\">";
  if ($none) {
    $html_string .= emit_option("null", $strings['generic']['none'], FALSE);
  }
  if ($all) {
    $html_string .= emit_option("all", $strings['generic']['all_users'], FALSE);
  }
  for ($i=0; $i < pg_num_rows($user_list); $i++) {
    $user = pg_fetch_row($user_list, $i);
    if (($user[0] != $po_user['id']) && 
	($po_user['type'] != PO_USER_TYPE_ADMIN) &&
	($user[3] == 't') &&
	($user[0] != $default_user_id)) {
      continue;
    }

    $html_string .= emit_option($user[0], "$user[1], $user[2]", $user[0] == $default_user_id);
  }
  $html_string .= "</select>";
  return $html_string;
}

function generate_html_clients_combo($identifier, $database, $owner_user_id, $current) {
  global $strings;

  $user_list = pg_query($database, "select users.identifier, last_name, first_name from users, client where users.identifier = client.client and client.users = $owner_user_id and client.trusted = 't' order by last_name, first_name");

  $html_string = "<select name=\"$identifier\" \">";
  $html_string .= emit_option("null", $strings['generic']['none'], FALSE);
  for ($i=0; $i < pg_num_rows($user_list); $i++) {
    $user = pg_fetch_row($user_list, $i);
    $html_string .= emit_option($user[0], "$user[1], $user[2]", $user[0] == $current);
  }
  $html_string .= "</select>";
  return $html_string;
}


?>
