<?php

//  Copyright (C) 2002-2006 Balint Kis (balint@k-i-s.net)
//  Copyright (C) 2005-2007 Solomon Peachy (pizza@shaftnet.org)

//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.

//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

function emit_DayEventHTML($event_id, $event_description, $start_date, $end_date, $view) {
  if (strlen($event_description) > 30)
    $event_description = sprintf("%0.30s", $event_description);

  $rval = "<table class=\"day_event\">\n";
  $rval .= "<tr>\n";
  $rval .= emit_td(emit_date_html($start_date, "time"));
  $rval .= emit_td(emit_a("event.php?event=$event_id&amp;view=$view", $event_description));
  $rval .= emit_td(emit_date_html($end_date, "time"));;
  $rval .= "</tr>\n";
  $rval .= "</table>\n";

  return $rval;
}

function emit_WeekEventHTML($event_id, $event_description, $start_date, $end_date, $view) {
  if (strlen($event_description) > 15)
    $event_description = sprintf("%0.15s", $event_description);
  
  $rval = emit_date_html($start_date, "time")." - ".emit_date_html($end_date, "time");

  $rval .= "<br/>";
  $rval .= emit_a("event.php?event=$event_id&amp;view=$view", $event_description);
  return $rval;
}

function emit_MonthEventHTML($event_id, $start_date, $end_date, $view) {
  return emit_a("event.php?event=$event_id&amp;view=$view", emit_date_html($start_date, "time"). " - ".emit_date_html($end_date, "time"));
}


function displayDayHTML($database, $user_id, $year, $month, $day, $cal_start_hour, $cal_end_hour, $view, $showcounts) {
  global $strings;
  
  $date = sprintf("%04d-%02d-%02d", $year, $month, $day);
  $result = pg_query($database, "select identifier, start_date, end_date, remark from calendar where users=$user_id and (start_date ~* '$date' or end_date ~* '$date') order by start_date");

  $num_of_events = pg_num_rows($result);

  for ($event = 0; $event < $num_of_events; $event++) {
    $event_data = pg_fetch_row($result, $event);
    $start_hour = (int)substr($event_data[1], 11, 2);
    $end_hour   = (int)substr($event_data[2], 11, 2);
    if ($start_hour < $cal_start_hour)
      $cal_start_hour = $start_hour;
    if ($end_hour > $cal_end_hour)
      $cal_end_hour = $end_hour;
  }

  $cells = $cal_end_hour - $cal_start_hour + 1;

  print "<div>";
  print "<table class=\"datebook\">\n";
  print "<tr>\n";
  $cell_width=floor(90/($cal_end_hour - $cal_start_hour + 1));
  for ($i = $cal_start_hour; $i <= $cal_end_hour; $i ++) {
    print emit_th("$i", "width=\"$cell_width%\"");
  }
  print emit_th(emit_a("event.add.php?year=$year&amp;month=$month&amp;day=$day", 
		       $strings['generic']['add']), "width=\"10%\" colspan=\"2\"");

  print "</tr>\n";

  for ($event = 0; $event < $num_of_events; $event++) {
    $event_data = pg_fetch_row($result, $event);
    $start_hour = substr($event_data[1], 11, 2);
    $end_hour   = substr($event_data[2], 11, 2);
    $end_minute = substr($event_data[2], 14, 2);
    print "<tr>\n";
    for ($i = $cal_start_hour; $i < $start_hour; $i ++)
      print emit_td("&nbsp;");

    $event_cells = $end_hour - $start_hour;
    if ($end_minute != 0)
      $event_cells++;

    print emit_td(emit_DayEventHTML($event_data[0], $event_data[3], $event_data[1], $event_data[2], $view), "class=\"selected\" colspan=\"$event_cells\"");

    for ($i = $end_hour; $i < $cal_end_hour; $i ++)
      print emit_td("&nbsp;");

    if (($end_hour != $start_hour) && ($end_minute == 0))
      print emit_td("&nbsp;");

    print emit_td(emit_a("event.edit.php?event=$event_data[0]", $strings['generic']['edit']));
    print emit_td(emit_a("event.del.php?event=$event_data[0]", $strings['generic']['delete']));
    print "</tr>";
  }

  if ($num_of_events == 0) {
    print "<tr>\n";
    $cells += 2;
    $message=$strings['datebook']['no_events'];
    print emit_th($message, "colspan=\"$cells\"");
    print "</tr>";    
  }
  print "</table>\n";
  print "</div>";
}

function displayWeekHTML($database, $user_id, $year, $week, $dayNames, $startDay, $view, $showcounts) {

  $firstDayOfWeek = getFirstDayOfWeek($year, $week, $startDay);
  print "<div>";
  print "<table class=\"datebook\">\n";
  print "<tr>\n";
  for ($dayIncrement = 0; $dayIncrement < 7; $dayIncrement++) {
    $curDate = adjustDay($firstDayOfWeek[0], $firstDayOfWeek[1], $firstDayOfWeek[2] + $dayIncrement);
    print emit_th(emit_a("my.datebook.php?view=1&amp;year=$curDate[0]&amp;month=$curDate[1]&amp;day=$curDate[2]\"", "<strong>" . $dayNames[($startDay+$dayIncrement)%7] . ", $curDate[2]</strong>"), 'width="14%"');
  }
  print "</tr>";
  print "<tr>\n";
  for ($dayIncrement = 0; $dayIncrement < 7; $dayIncrement++) {
    $num_photos = 0;
    $curDate = adjustDay($firstDayOfWeek[0], $firstDayOfWeek[1], $firstDayOfWeek[2] + $dayIncrement);
    if ($showcounts) {
      $num_photos = photos_between_dates($database, $user_id, 
					 mktime(0,0,0,$curDate[1],$curDate[2],$curDate[0]),
					 mktime(0,0,0,$curDate[1],$curDate[2]+1,$curDate[0]));
    }

    $today = getdate(time());
    if ($curDate[0] == $today["year"] && $curDate[1] == $today["mon"] && $curDate[2] == $today["mday"])
      $bgcolor = "class=\"selected\"";
    else
      $bgcolor = "";

    $string = "";
    if ($showcounts && $num_photos > 0) {
      $string .= "[$num_photos]<br/>";
    }

    $date = sprintf("%04d-%02d-%02d", $curDate[0], $curDate[1], $curDate[2]);
    $result = pg_query($database, "select identifier, start_date, end_date, remark from calendar where users=$user_id and (start_date ~* '$date' or end_date ~* '$date') order by start_date");
    $num_of_events = pg_num_rows($result);
    for ($event = 0; $event < $num_of_events; $event++) {
      $event_data = pg_fetch_row($result, $event);
      $string .= emit_WeekEventHTML($event_data[0], $event_data[3], $event_data[1], $event_data[2], $view);
    }
    $string .= "&nbsp;";

    print emit_td($string, $bgcolor);
  }

  print "</tr>\n";
  print "</table>\n";
  print "</div>";
}

function displayMonthHTML($database, $user_id, $year, $month, $dayNames, $startDay, $view, $showcounts) {
  global $strings;

  print "<div>";
  print "<table class=\"datebook\">\n";
  print "<tr>\n";
  print emit_th($strings['generic']['week'], "width=\"2%\"");
  for ($dayIncrement = 0; $dayIncrement < 7; $dayIncrement++)
    print emit_th("<strong>".$dayNames[($startDay + $dayIncrement) % 7]."</strong>", "width=\"14%\"");
  print "</tr>\n";

  // get timestamp of first day of month at noon
  $timestamp = mktime(12, 0, 0, $month, 1, $year);
  // get first day index 0 Sunday
  $date = getdate($timestamp);
  $day = $startDay + 1 - $date["wday"];

  if ($day > 1)
    $day -= 7;

  $today = getdate(time());
  $daysInMonth = getDaysInMonth($year, $month);

  while ($day <= $daysInMonth) {
    print "<tr>\n";
    $curDate = adjustDay($year, $month, $day);
    $weekNumber=getWeekNumber($curDate[0], $curDate[1], $curDate[2], $startDay);
    print emit_td(emit_a("my.datebook.php?view=2&amp;year=$year&amp;month=$curDate[1]&amp;day=$curDate[2]", "<strong>$weekNumber</strong>"));
    for ($i = 0; $i < 7; $i++) {
      $curDate = adjustDay($year, $month, $day);
      $num_photos = 0;
      if ($showcounts) {
	$num_photos = photos_between_dates($database, $user_id, 
					   mktime(0,0,0,$curDate[1],$curDate[2],$curDate[0]),
					   mktime(0,0,0,$curDate[1],$curDate[2]+1,$curDate[0]));
      }

      $bgcolor = "";
      if ($curDate[2]!=$day)
	$bgcolor = "class=\"inactive\"";

      if (($year == $today["year"]) && 
	  ($month == $today["mon"]) && 
	  ($day == $today["mday"]))
	$bgcolor = "class=\"selected\"";

      $string = "";

      $bgcolor .= " align=\"left\" ";

      $string .= emit_a("my.datebook.php?view=1&amp;year=$curDate[0]&amp;month=$curDate[1]&amp;day=$curDate[2]", "<strong>$curDate[2]</strong>");
      if ($showcounts && $num_photos > 0) {
	$string .= "&nbsp;&nbsp;[$num_photos]";
      }
      $string .= "<br/>";
      $date = sprintf("%04d-%02d-%02d", $curDate[0], $curDate[1], $curDate[2]);
      $result = pg_query($database, "select identifier, start_date, end_date from calendar where users=$user_id and (start_date ~* '$date' or end_date ~* '$date') order by start_date");
      $num_of_events = pg_num_rows($result);
      for ($event = 0; $event < $num_of_events; $event++) {
        $event_data = pg_fetch_row($result, $event);
        $string .= emit_MonthEventHTML($event_data[0], $event_data[1], $event_data[2], $view);
      }
      print emit_td($string, $bgcolor);
      $day++;
    }
    print "</tr>\n";
  }

  print "</table>\n";
  print "</div>";
}

?>
