
//  Copyright (C) 2005 Balint Kis (balint@k-i-s.net)

//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.

//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.applet.Applet;
import java.net.*;
import java.io.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;
import javax.swing.border.*;
import javax.swing.colorchooser.*;
import javax.swing.filechooser.*;
import javax.accessibility.*;

class BButton {
    int offsetX, offsetY;
    int height, width;
    boolean isPressed;
    boolean isHighLighted;
    Image image[];
    Applet applet;

    public BButton(Applet applet, int offsetX, int offsetY, int width, int height, String fileName) {
        this.offsetX = offsetX;
        this.offsetY = offsetY;
        this.height = height;
        this.width = width;
        this.isPressed = false;
        this.isHighLighted = false;
        this.applet = applet;

        image = new Image[4];
        image[0] = applet.getImage(applet.getDocumentBase(), fileName + ".normal.gif");
        image[1] = applet.getImage(applet.getDocumentBase(), fileName + ".pressed.gif");
        image[2] = applet.getImage(applet.getDocumentBase(), fileName + ".highlighted.gif");
        image[3] = applet.getImage(applet.getDocumentBase(), fileName + ".disabled.gif");
    }

    public boolean isInFocus(int cx, int cy) {
        return ((cx > offsetX) && (cx < (offsetX + width)) &&
                (cy > offsetY) && (cy < (offsetY + height)));
    }

    public void setHighLight(boolean isHighLighted) {
        this.isHighLighted = isHighLighted;
        if (!isHighLighted)
            isPressed = false;
    }

    public void setStatus(boolean isPressed) {
        this.isPressed = isPressed;
    }

    public void paint(Graphics g) {
        if (isPressed) {
            g.drawImage(image[1], offsetX, offsetY, applet);
            return;
        }
        if (isHighLighted) {
            g.drawImage(image[2], offsetX, offsetY, applet);
            return;
        }
        g.drawImage(image[0], offsetX, offsetY, applet);
    }
}


class Cell {
    boolean isHighLighted;
    Color color;

    Cell(boolean isHighLighted, Color color) {
        this.isHighLighted = isHighLighted;
        this.color = color;
    }
}

class ColorHistory {
    Color color[];
    int offsetX, offsetY;
    int currentColorIndex;
    Dimension dimension;

    ColorHistory(int offsetX, int offsetY, Dimension dimension) {
        this.offsetX = offsetX;
        this.offsetY = offsetY;
        this.dimension = dimension;
        /* create the initial colormap */
        color = new Color[9];
        color[0] = Color.black;
        color[1] = Color.white;
        color[2] = Color.red;
        color[3] = Color.green;
        color[4] = Color.blue;
        color[5] = Color.yellow;
        color[6] = Color.cyan;
        color[7] = Color.magenta;
        color[8] = Color.orange;

        currentColorIndex = 0;
    }

    public void Display(Graphics g) {
        /* draw color cells */
        for (int x = 0; x < 9; x++) {
            g.setColor(color[x]);
            g.fillRect(offsetX + x * dimension.width,
                       offsetY,
                       dimension.width,
                       dimension.height);
        }
        /* draw grids */
        g.setColor(Color.gray);

        g.drawLine(offsetX, offsetY, offsetX+dimension.width*9, offsetY);
        for (int x = 0; x < 10; x++)
            g.drawLine(offsetX + x * dimension.width, offsetY, offsetX + x * dimension.width, offsetY+dimension.height);
        g.drawLine(offsetX, offsetY+dimension.height, offsetX+dimension.width*9, offsetY+dimension.height);

        g.setColor(color[currentColorIndex]);
        int x1 = offsetX - 3 + currentColorIndex * dimension.width;
        int y1 = offsetY - 3;
        int x2 = dimension.width + 7;
        int y2 = dimension.height + 7;
        g.fillRect(x1, y1, x2, y2);

        x2 += x1 - 1;
        y2 += y1 - 1;

        g.setColor(Color.gray);
        g.drawLine(x1, y1, x1, y2);
        g.drawLine(x2, y1, x2, y2);
        g.drawLine(x1, y1, x2, y1);
        g.drawLine(x1, y2, x2, y2);
    }

    public void isInFocus(int cx, int cy) {
        for (int index = 0; index < 9; index++) {
            if ((cx > (offsetX + index * dimension.width)) && (cx < (offsetX + (index+1) * dimension.width)) &&
                (cy > (offsetY)) && (cy < (offsetY + dimension.height))) {
                currentColorIndex = index;
            }
        }
    }

    public Color getColor() {
        return color[currentColorIndex];
    }

    public void setColor(Color newColor) {
        color[currentColorIndex] = newColor;
    }
}

public class LayoutSearch extends Applet implements Runnable, MouseListener, MouseMotionListener {
    Thread canvasPainter;
    Image     offScreen;
    Dimension offScreensize;
    Graphics  offGraphics;

    int numOfCellsX;
    int numOfCellsY;
    Dimension dimensionCanvas;
    Dimension dimensionCell;
    Cell cell[][];
    ColorHistory colorHistory;

    BButton buttonClear;
    BButton buttonColorChooser;
    BButton buttonSubmit;

    String imagePath = null;
    String submitStringURL = null;
    URL    submitAnchorURL = null;

    public void init() {
        String parameter;

        numOfCellsX = 10;
        numOfCellsY = 10;
        dimensionCanvas = new Dimension(400,400);

        if ((parameter = getParameter("canvas_width")) != null)
            dimensionCanvas.width = Integer.parseInt(parameter);

        if ((parameter = getParameter("canvas_height")) != null)
            dimensionCanvas.height = Integer.parseInt(parameter);

        if ((parameter = getParameter("horizontal_cell_count")) != null)
            numOfCellsX = Integer.parseInt(parameter);

        if ((parameter = getParameter("vertical_cell_count")) != null)
            numOfCellsY = Integer.parseInt(parameter);

        if ((imagePath = getParameter("image_path")) == null)
            imagePath = "";

        submitStringURL = getParameter("submit_url");
        if (submitStringURL != null) {
            try {
                submitAnchorURL = new URL(submitStringURL);
            } catch (MalformedURLException e) {
                System.err.println("LayoutSearch: Parse error: " + e);
            }
        }

        buttonClear        = new BButton(this, 10, dimensionCanvas.height - 30, 103, 25, getCodeBase() + imagePath + "/clear");
        buttonColorChooser = new BButton(this, 143, dimensionCanvas.height - 30, 84, 25, getCodeBase() + imagePath + "/color.chooser");
        buttonSubmit       = new BButton(this, dimensionCanvas.width - 68, dimensionCanvas.height - 30, 58, 25, getCodeBase() + imagePath + "/submit");

        colorHistory = new ColorHistory(237, dimensionCanvas.height - 30, new Dimension(24, 24));

        dimensionCanvas.height -= 30;
        dimensionCanvas.height = (dimensionCanvas.height / numOfCellsY) * numOfCellsY;
        dimensionCanvas.width  = (dimensionCanvas.width  / numOfCellsX) * numOfCellsX;

        dimensionCell = new Dimension(dimensionCanvas.width / numOfCellsX, dimensionCanvas.height / numOfCellsY);
        cell = new Cell[numOfCellsX][numOfCellsY];

        for (int y = 0; y < numOfCellsY; y++)
            for (int x = 0; x < numOfCellsX ; x++) {
                int index = y * numOfCellsX + x;
                String colorString = getParameter("px" + index);
                if ((colorString == null) || (colorString == "")) {
                    cell[x][y] = new Cell(false, Color.white);
                    continue;
                }
                colorString = colorString.replaceAll("#", "");
                Color currentColor = new Color(Integer.parseInt(colorString, 16));
                cell[x][y] = new Cell(false, currentColor);
            }

        addMouseListener(this);
    }

    public void destroy() {
        removeMouseListener(this);
    }

    public void start() {
        canvasPainter = new Thread(this);
        canvasPainter.start();
    }

    public synchronized void stop() {
        canvasPainter = null;
        notify();
    }

    public void clearCanvas() {
        for (int x = 0; x < numOfCellsX ; x++)
            for (int y = 0; y < numOfCellsY; y++) {
                cell[x][y].color = Color.white;
            }
    }

    public String getCanvasContentString() {
        String parameters = "submit=yes" + "&width=" + numOfCellsX + "&height=" + numOfCellsY;
        for (int y = 0; y < numOfCellsY; y++)
            for (int x = 0; x < numOfCellsX ; x++) {
                int red   = cell[x][y].color.getRed();
                int green = cell[x][y].color.getGreen();
                int blue  = cell[x][y].color.getBlue();
                int index = y * numOfCellsX + x;
                parameters = parameters + "&px" + index + "=" + Integer.toHexString(blue + 256 * (green + 256 * red));
            }
        return parameters;
    }

    public void submitSearch(URL submitAnchorURL) {
        try {
            String parameters = getCanvasContentString();
            URL submitURL = new URL(submitAnchorURL + "?" + parameters);
            getAppletContext().showDocument(submitURL);
        } catch (Exception e) {
            System.err.println("LayoutSearch: HTTP Post Error: " + e);
        }
    }

    public void setCellColor(int x, int y, Color color) {
        int cellIdxX = numOfCellsX * (x - 1) / dimensionCanvas.width;
        int cellIdxY = numOfCellsY * (y - 1) / dimensionCanvas.height;

        if (!((cellIdxX >= 0) && (cellIdxX < numOfCellsX) &&
              (cellIdxY >= 0) && (cellIdxY < numOfCellsY)))
            return;

        cell[cellIdxX][cellIdxY].color = color;
    }

    public void Display(Graphics g) {
        /* draw canvas cells */
        for (int x = 0; x < numOfCellsX; x++)
            for (int y = 0; y < numOfCellsY; y++) {
                g.setColor(cell[x][y].color);
                g.fillRect(x * dimensionCell.width,
                           y * dimensionCell.height,
                           dimensionCell.width,
                           dimensionCell.height);
            }
        /* draw grids */
        g.setColor(Color.gray);
        for (int x = 0; x < numOfCellsX; x++)
            g.drawLine(x * dimensionCell.width, 0, x * dimensionCell.width, dimensionCell.height * numOfCellsY);
        g.drawLine(dimensionCell.width * numOfCellsX, 0, dimensionCell.width * numOfCellsX, dimensionCell.height * numOfCellsY);
        for (int y = 0; y < numOfCellsY; y++)
            g.drawLine(0, y * dimensionCell.height, dimensionCell.width * numOfCellsX, y * dimensionCell.height);
        g.drawLine(0, dimensionCell.height * numOfCellsY, dimensionCell.width * numOfCellsX, dimensionCell.height * numOfCellsY);
    }

    public synchronized void startAnimation() {
        notify();
    }

    public void run() {
        Thread me = Thread.currentThread();
        while (canvasPainter == me) {
            repaint();
            try {
                Thread.sleep(100);
            }
            catch (InterruptedException e) {
                break;
            }
        }
    }

    public synchronized void update(Graphics graphics) {
        Dimension d = getSize();
        if ((offScreen == null) || (d.width != offScreensize.width) || (d.height != offScreensize.height)) {
            offScreen = createImage(d.width, d.height);
            offScreensize = d;
            offGraphics = offScreen.getGraphics();
            offGraphics.setFont(getFont());
        }

        offGraphics.setColor(Color.white);
        offGraphics.fillRect(0, 0, d.width, d.height);
        Display(offGraphics);
        buttonClear.paint(offGraphics);
        buttonColorChooser.paint(offGraphics);
        buttonSubmit.paint(offGraphics);
        colorHistory.Display(offGraphics);
        graphics.drawImage(offScreen, 0, 0, null);
    }

    public void mousePressed(MouseEvent e) {
        addMouseMotionListener(this);
        buttonClear.setStatus(buttonClear.isInFocus(e.getX(), e.getY()));
        buttonColorChooser.setStatus(buttonColorChooser.isInFocus(e.getX(), e.getY()));
        buttonSubmit.setStatus(buttonSubmit.isInFocus(e.getX(), e.getY()));
        mouseClicked(e);
    }

    public void mouseDragged(MouseEvent e) {
        mouseClicked(e);
    }

    public void mouseClicked(MouseEvent e) {
        setCellColor(e.getX(), e.getY(), colorHistory.getColor());
        repaint();
        e.consume();
    }

    public void mouseReleased(MouseEvent e) {
        removeMouseMotionListener(this);

        if (buttonClear.isInFocus(e.getX(), e.getY())) {
            buttonClear.setStatus(false);
            clearCanvas();
        }
        if (buttonColorChooser.isInFocus(e.getX(), e.getY())) {
            buttonColorChooser.setStatus(false);
            colorHistory.setColor(JColorChooser.showDialog(this, "Select Color", colorHistory.getColor()));
        }
        if (buttonSubmit.isInFocus(e.getX(), e.getY())) {
            buttonSubmit.setStatus(false);
            if (submitAnchorURL != null) {
                submitSearch(submitAnchorURL);
            }
        }

        colorHistory.isInFocus(e.getX(), e.getY());
        repaint();
        e.consume();
    }

    public void mouseEntered(MouseEvent e) {
        mouseMoved(e);
    }

    public void mouseExited(MouseEvent e) {
        mouseMoved(e);
    }

    public void mouseMoved(MouseEvent e) {
        buttonClear.setHighLight(buttonClear.isInFocus(e.getX(), e.getY()));
        buttonColorChooser.setHighLight(buttonColorChooser.isInFocus(e.getX(), e.getY()));
        buttonSubmit.setHighLight(buttonSubmit.isInFocus(e.getX(), e.getY()));
        e.consume();
    }


    public String getAppletInfo() {
	return "Photo Organizer Layout Search Client v1.0 \n Copyright (C) 2005, Balint Kis (balint@k-i-s.net)";
    }

    public String[][] getParameterInfo() {
	String[][] info = {
	  {"canvas_width",          "int", "width of the canvas in pixels"},
	  {"canvas_height",         "int", "height of the canvas in pixels"},
            {"horizontal_cell_count", "int", "number of horizontal cells"},
            {"vertical_cell_count",   "int", "number of vertical cells"},
            {"image_path",            "URL", "url pointing to the images"},
            {"submit_url",            "URL", "submit url"}
	};
	return info;
    }
}
